<?php
/**
 * YANA library
 *
 * @package yana
 * @license http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * @ignore
 */

/**
 * utility DisplayUtility
 *
 * This is a utility class. It encapsulates extensions to use with
 * the smarty temlate engine.
 *
 * {@internal
 *
 * Additional smarty functions and modifiers are documented elsewhere.
 * This file should be ignored in API-documentation.
 *
 * }}
 *
 * @static
 * @access      public
 * @package     yana
 * @subpackage  core
 *
 * @ignore
 */
class DisplayUtility extends Utility
{

    /**
     * toTimestamp
     *
     * converts input created by usage of the smarty functions
     * "html_select_time" and "html_select_date" to a UNIX-Timestamp
     *
     * @access  public
     * @static
     * @param   string  $prefix
     * @param   array   $ARGS
     * @return  int
     */
    function toTimestamp($prefix, $ARGS)
    {
        return mktime((int) @$ARGS[$prefix.'hour'], (int) @$ARGS[$prefix.'minute'], (int) @$ARGS[$prefix.'second'], (int) @$ARGS[$prefix.'month'], (int) @$ARGS[$prefix.'day'], (int) @$ARGS[$prefix.'year']);
    }

    /**
     * smarty function guiCreate
     *
     * <pre>
     * This function takes the following arguments:
     *
     * string  $template        (mandatory) template that should be used (view_details, view_seperated, view, new, edit, search)
     * string  $file            (mandatory) path and name of structure file
     * string  $table           (mandatory) table to choose from structure file
     * string  $where           (optional)  sequence for SQL-where clause <FIELDNAME1>=<VALUE1>[,<FIELDNAME2>=<VALUE2>[,...]]
     * string  $sort            (optional)  name of column to sort entries by
     * boolean $desc            (optional)  sort entries descendant (true) or ascendant (false)
     * integer $page            (optional)  number of 1st entry to show
     * integer $entries         (optional)  number of entries to show
     * boolean $titles          (optional)  should fieldnames be shown (true=yes, false=no)
     * string  $on_new          (optional)  name of action (plugin-function) to activate in order to compute the event
     * string  $on_edit         (optional)  name of action (plugin-function) to activate in order to compute the event
     * string  $on_delete       (optional)  name of action (plugin-function) to activate in order to compute the event
     * string  $on_search       (optional)  name of action (plugin-function) to activate in order to compute the event
     * boolean $advanced_search (optional)  should advanced search form be shown (true=yes, false=no)
     * </pre>
     *
     * @access  public
     * @static
     * @param   array   $params
     * @param   Smarty  &$smarty
     * @return  string
     */
    function guiCreate($params, &$smarty)
    {
        global $YANA;
        /*
         * Parameters 'on_edit', 'on_delete', 'on_new', 'on_search'
         */
        settype($params['on_edit'],   "string");
        settype($params['on_delete'], "string");
        settype($params['on_new'],    "string");
        settype($params['on_search'], "string");
        $on_edit   = strtolower($params['on_edit']);
        $on_delete = strtolower($params['on_delete']);
        $on_new    = strtolower($params['on_new']);
        $on_search = strtolower($params['on_search']);
        $YANA->setVar('ON_EDIT',     $on_edit);
        $YANA->setVar('ON_DELETE',   $on_delete);
        $YANA->setVar('ON_NEW',      $on_new);
        $YANA->setVar('ON_SEARCH',   $on_search);

        /*
         * Parameter 'table'
         */
        settype($params['table'], "string");
        $table = strtolower($params['table']);

        /*
         * Parameter 'sort'
         */
        if (!isset($_SESSION['sort']) || !is_array($_SESSION['sort'])) {
            $_SESSION['sort'] = array();
        }
        if (isset($_GET['sort'])) {
            $sort = $_GET['sort'];
        } else if (isset($_SESSION['sort'][$table])) {
            $sort = $_SESSION['sort'][$table];
        } else {
            $sort = $params['sort'];
        }
        settype($sort, "string");

        /*
         * Parameter 'where'
         */
        if (!isset($_SESSION['where']) || !is_array($_SESSION['where'])) {
            $_SESSION['where'] = array();
        }
        if (isset($params['where'])) {
            if (is_string($params['where']) || is_array($params['where'])) {
                $where = $params['where'];
            } else {
                $where = array();
            }
        } else {
            $where = array();
        }

        /*
         * Parameter 'template'
         */
        settype($params['template'], "string");
        $template = strtolower($params['template']);
        $YANA->setVar('NAMESPACE', $template);

        /*
         * Parameter 'file'
         */
        settype($params['file'], "string");
        $file = $params['file'];

        /*
         * Parameter 'titles'
         */
        if (isset($params['titles'])) {
            if ($params['titles'] == 'hidden' || $params['titles'] == 'false' || (bool) $params['titles'] == false) {
                $show_titles = false;
            } else {
                $show_titles = true;
            }
        } else {
            $show_titles = true;
        }

        /*
         * Parameter 'desc'
         */
        if (!isset($_SESSION['desc']) || !is_array($_SESSION['desc'])) {
            $_SESSION['desc'] = array();
        }
        if (isset($_GET['desc'])) {
            $desc = ($_GET['desc']) ? true : false;
        } else if (isset($_SESSION['desc'][$table])) {
            $desc = $_SESSION['desc'][$table];
        } else {
            $desc = $params['desc'];
        }
        settype($desc, "boolean");
        $_SESSION['desc'][$table] = $desc;
        $YANA->setVar('DESC', $desc);

        /*
         * Parameter 'advanced_search'
         */
        if (!isset($_SESSION['advanced_search']) || !is_array($_SESSION['advanced_search'])) {
            $_SESSION['advanced_search'] = array();
        }
        if (isset($_GET['advanced_search'])) {
            $advanced_search = ($_GET['advanced_search']) ? true : false;
        } else if (isset($_SESSION['advanced_search'][$table])) {
            $advanced_search = ($_SESSION['advanced_search'][$table]) ? true : false;
        } else {
            $advanced_search = $params['advanced_search'];
        }
        settype($advanced_search, "boolean");
        $_SESSION['advanced_search'][$table] = $advanced_search;
        $YANA->setVar('ADVANCED_SEARCH', $advanced_search);

        /*
         * Parameter 'page'
         */
        if (!isset($_SESSION['page']) || !is_array($_SESSION['page'])) {
            $_SESSION['page'] = array();
        }
        if (isset($_GET['page'])) {
            $page = $_GET['page'];
        } else if (isset($_SESSION['page'][$table])) {
            $page = $_SESSION['page'][$table];
        } else {
            $page = $params['page'];
        }
        settype($page, "integer");
        if ($page < 0) {
            $page = 0;
        }
        $_SESSION['page'][$table] = $page;
        $YANA->setVar('PAGE',       $page);
        $YANA->setVar('FIRST_PAGE', $page+1);

        /*
         * Parameter 'entries'
         */
        if (!isset($_SESSION['entries']) || !is_array($_SESSION['entries'])) {
            $_SESSION['entries'] = array();
        }
        if (isset($_GET['entries'])) {
            $entries = $_GET['entries'];
        } else if (isset($_SESSION['entries'][$table])) {
            $entries = $_SESSION['entries'][$table];
        } else {
            $entries = $params['entries'];
        }
        settype($entries, "integer");
        if ($entries < 1) {
            $entries = 5;
        }
        $_SESSION['entries'][$table] = $entries;
        $YANA->setVar('ENTRIES', $entries);

        /*
         * Begin business logic
         */
        $document = new smartTemplate();
        $primary_key = "";
        $document->insertByReference('PRIMARY_KEY',       $primary_key);
        $document->insertByReference('TABLE',             $table);
        $document->insertByReference('SHOW_FIELD_TITLES', $show_titles);

        $db = $YANA->connect($file);
        $db->read();

        if ($db->exists()) {
            $last_page = $db->length($table);
            if ($page > $last_page) {
                if ($last_page > 0) {
                    $page = $last_page - 1;
                    settype($page, 'int');
                } else {
                    $page = 0;
                }
                $YANA->setVar('PAGE',       $page);
                $YANA->setVar('FIRST_PAGE', $page+1);
            }
            $primary_key = $db->structure->getPrimaryKey($table);

            /*
             * Parameter 'sort'
             */
            if (empty($sort) || !$db->structure->isColumn($table,$sort)) {
                $sort = $primary_key;
            }
            $_SESSION['sort'][$table] = $sort;
            $YANA->setVar('SORT', $sort);

            $content =& $db->structure->get("TABLES.${table}.CONTENT");
            $document->insertByReference('CONTENT', $content);

            /* resolve foreign keys */
            assert('!isset($key); /* cannot redeclare variable $key */');
            foreach (array_keys($content) as $key)
            {
                /* if is foreign key of type select ... */
                if ($db->structure->isForeignKey($table, $key) && strcasecmp($db->structure->getType($table, $key), 'select') == 0) {
                    $default = $db->structure->getDefault($table, $key);
                    /* check where the foreign key points to */
                    assert('!isset($foreign_keys);');
                    $foreign_keys = $db->structure->getForeignKeys($table);
                    assert('!isset($foreign_table);');
                    $foreign_table = $foreign_keys[strtoupper($key)];
                    /* if default is not set */
                    if (is_null($default)) {
                        assert('!isset($defaults);');
                        $defaults = $db->get("${foreign_table}.*.${key}");
                        $content[strtoupper($key)]['DEFAULT'] = array();
                        assert('!isset($default);');
                        foreach ($defaults as $default)
                        {
                            $content[strtoupper($key)]['DEFAULT'][$default] = $default;
                        }
                        assert('is_array($content[strtoupper($key)]["DEFAULT"]);');
                        unset($default, $defaults);
                    /* or default is an array of table references */
                    } else if (is_array($default) && count($default) === 1) {
                        assert('!isset($col1) && !isset($col2) && !isset($defaultKeys) && !isset($defaultValues);');
                        foreach ($default as $col1 => $col2)
                        {
                            if ($db->structure->isColumn($foreign_table, $col1)) {
                                $defaultKeys = $db->get("${foreign_table}.*.${col1}");
                            } else {
                                $defaultKeys = array();
                                trigger_error("The foreign key '${key}' in table '${table}' references a non-existing column '${col1}' in table '${foreign_table}'.",E_USER_ERROR);
                            }
                            if ($db->structure->isColumn($foreign_table, $col2)) {
                                $defaultValues = $db->get("${foreign_table}.*.${col2}");
                            } else {
                                $defaultValues = array();
                                trigger_error("The foreign key '${key}' in table '${table}' references a non-existing column '${col2}' in table '${foreign_table}'.",E_USER_ERROR);
                            }
                            break;
                        } /* end foreach */
                        /* check unique constraint */
                        if (count($defaultKeys) !== count(array_unique($defaultKeys))) {
                            trigger_error("The column '${col1}' in table '${foreign_table}' referenced as foreign key from table '${table}' contains ambigious values. Possible reasons are that your data is inconsistent or your database scheme contains errors.", E_USER_WARNING);
                        }
                        unset($col1,$col2);
                        assert('is_array($defaultKeys) && is_array($defaultValues) && count($defaultKeys) === count($defaultValues);');
                        assert('!isset($i); /* Unable to redeclare $i */');
                        $content[strtoupper($key)]['DEFAULT']  = array();
                        assert('empty($content[strtoupper($key)]["DEFAULT"]);');
                        /* array_combine() became available in PHP 5, a workaround is required for PHP 4 */
                        for ($i=0; $i<count($defaultKeys); $i++)
                        {
                            $content[strtoupper($key)]['DEFAULT'][$defaultKeys[$i]] = $defaultValues[$i];
                        }
                        unset($defaultKeys,$defaultValues);
                    }
                    unset($foreign_table, $foreign_keys);
                } /* end if */
            } /* end foreach */
            unset($key);

            switch ($template)
            {
                case 'view_details': case 'view_seperated': case 'view': case 'edit':
                    $YANA->setVar('LAST_PAGE', $last_page);
                    switch ($template)
                    {
                        case 'view_details':
                            $document->setFilename($YANA->skin->getFile("gui_view_details"));
                        break;
                        case 'view_seperated':
                            $document->setFilename($YANA->skin->getFile("gui_view_seperated"));
                        break;
                        case 'view':
                            $document->setFilename($YANA->skin->getFile("gui_view"));
                        break;
                        case 'edit':
                            $document->setFilename($YANA->skin->getFile("gui_edit"));
                        break;
                    } /* end switch */
                    /* Note:
                     * when searching for a primary key, page numbering is irrelevant
                     * so it needs to be reseted
                     */
                    if (!empty($where)) {
                        if (is_string($where)) {
                            if (strpos(strtolower($where),strtolower($primary_key).'=') > -1) {
                                $page = 0;
                            }
                        } else if (is_array($where)){
                            /* Note:
                             * This array scan is not much of a performance issue,
                             * as $where has in up to 90% of all cases at most
                             * 1 item.
                             */
                            assert('!isset($temp);');
                            foreach ($where as $temp)
                            {
                                if (isset($temp[0]) && strcasecmp($temp[0],$primary_key) === 0) {
                                    $page = 0;
                                    break;
                                }
                            } /* end foreach */
                            unset($temp);
                        } /* end if */
                    } /* end if */
                    $ROWS = $db->get($table, $where,(string)$sort,$page,$entries,(bool)$desc);
                    $CURRENT_INDEX = array_pop(array_keys($ROWS));
                    /* resolve page number */
                    if (!empty($where)) {
                        assert('!isset($temp);');
                        $temp = $db->get("$table.*.$primary_key",'',(string)$sort,0,0,(bool)$desc);
                        assert('is_array($temp);');
                        $temp = array_search($CURRENT_INDEX, $temp);
                        assert('is_int($temp);');
                        $_SESSION['page'][$table] = $temp;
                        $YANA->setVar('PAGE', $temp);
                        unset($temp);
                    }
                    $document->insert('CURRENT_INDEX', $CURRENT_INDEX);
                    $document->insert('LIST_OF_ENTRIES', DisplayUtility::_calcFooter($page,$last_page,$entries));
                    $document->insert('ROWS', $ROWS);
                break;

                case 'new':
                    $document->setFilename($YANA->skin->getFile("gui_new"));
                break;

                case 'search':
                    if ($advanced_search) {
                        $document->setFilename($YANA->skin->getFile("gui_search"));
                    } else {
                        $document->setFilename($YANA->skin->getFile("gui_small_search"));
                    }
                    if ($entries > 1) {
                        $document->insert('HITLIST_TEMPLATE', ($YANA->skin->getFile("gui_view")));
                    } else {
                        $document->insert('HITLIST_TEMPLATE', ($YANA->skin->getFile("gui_view_details")));
                    }
                    $where = array();
                    assert('!isset($has_where_clause); /* cannot redeclare variable $has_where_clause */');
                    $has_where_clause = false;
                    assert('!isset($fieldname); /* cannot redeclare variable $fieldname */');
                    foreach (array_keys($content) as $fieldname)
                    {
                        $fieldname = strtoupper($fieldname);
                        if ((!$advanced_search || $_POST[strtolower('is_active_'.$fieldname)] === 'true')) {
                            $has_where_clause = true;
                            /* for TYPE=time create range */
                            if (strcasecmp($db->structure->getType($table,$fieldname),'time') === 0) {

                                assert('!isset($time) && !isset($time_stamp);');

                                global $YANA;

                                /* start time*/
                                $time = array();
                                $time['Y'] = (int) $YANA->getRequestVar("${fieldname}_START_Year");
                                $time['M'] = (int) $YANA->getRequestVar("${fieldname}_START_Month");
                                $time['D'] = (int) $YANA->getRequestVar("${fieldname}_START_Day");
                                $time['h'] = (int) $YANA->getRequestVar("${fieldname}_START_Hour");
                                $time['m'] = (int) $YANA->getRequestVar("${fieldname}_START_Minute");
                                $time['s'] = (int) $YANA->getRequestVar("${fieldname}_START_Second");
                                if (!empty($time)) {
                                    $time_stamp = mktime($time['h'], $time['m'], $time['s'], $time['M'], $time['D'], $time['Y']);
                                    if (!empty($time_stamp) && is_int($time_stamp)) {
                                        array_push($where, array($fieldname, $time_stamp, '>='));
                                    }
                                }

                                /* end time*/
                                $time = array();
                                $time['Y'] = (int) $YANA->getRequestVar("${fieldname}_END_Year");
                                $time['M'] = (int) $YANA->getRequestVar("${fieldname}_END_Month");
                                $time['D'] = (int) $YANA->getRequestVar("${fieldname}_END_Day");
                                $time['h'] = (int) $YANA->getRequestVar("${fieldname}_END_Hour");
                                $time['m'] = (int) $YANA->getRequestVar("${fieldname}_END_Minute");
                                $time['s'] = (int) $YANA->getRequestVar("${fieldname}_END_Second");
                                if (!empty($time)) {
                                    if (empty($time['h'])) {
                                        $time['h'] = 23;
                                    }
                                    if (empty($time['m'])) {
                                        $time['m'] = 59;
                                    }
                                    if (empty($time['s'])) {
                                        $time['s'] = 59;
                                    }
                                    $time_stamp = mktime($time['h'], $time['m'], $time['s'], $time['M'], $time['D'], $time['Y']);
                                    if (!empty($time_stamp) && is_int($time_stamp)) {
                                        array_push($where, array($fieldname, $time_stamp, '<='));
                                    }
                                }
                                unset($time,$time_stamp);

                            /* for other create match */
                            } else {
                                if (!empty($_POST[strtoupper($fieldname)])) {
                                    array_push($where, array($fieldname, '%'.$_POST[strtoupper($fieldname)].'%', 'like'));
                                }
                            }
                        }
                    } /* end foreach */
                    unset($fieldname);
                    if (!$has_where_clause) {
                        if (!empty($_SESSION['where'][$table])) {
                            $where = $_SESSION['where'][$table];
                        }
                    } else {
                        $_SESSION['where'][$table] = $where;
                    }
                    $document->insert('IS_SEARCH', true);
                    $ROWS = $db->get($table, $where,(string)$sort,(int)$page,(int)$entries,(bool)$desc);
                    $document->insert('ROWS', $ROWS);
                    $last_page = $db->length($table, $where);
                    $YANA->setVar('LAST_PAGE', $last_page);
                    $document->insertByReference('LIST_OF_ENTRIES', DisplayUtility::_calcFooter($page,$last_page,$entries));
                break;

                default:
                    trigger_error("Template-Syntaxerror, unknown template for GUI-Generator: \"".$template."\"", E_USER_WARNING);
                    return "";
                break;
            } /* end switch */
            if (isset($ROWS)) {
                $YANA->setVar('OFFSET_PAGE', $page + count($ROWS));
            }
            $result = $document->get();
        } else {
            trigger_error("Template-Syntaxerror, database connection failed", E_USER_WARNING);
            return "";
        }

        return $result;

    }

    /**
     * _calcFooter
     *
     * @access private
     * @param  int $current_page
     * @param  int $last_page
     * @param  int $no_of_entries
     * @return array
     */
    function _calcFooter($current_page,$last_page,$entries_per_page)
    {
        $list_of_pages = array();
        if ($last_page > ($entries_per_page*2)) {
            $dots = false;
            $is_too_long = ($last_page>(10*$entries_per_page));
            for($k=0;$k<ceil($last_page/$entries_per_page);$k++)
            {
                /**
                 * if more than 10 pages exist and current page is not first page or last page
                 * and is not current page or previous or next 3 pages
                 */
                $is_near_current = ($k>(floor($current_page/$entries_per_page)-3) && $k<(floor($current_page/$entries_per_page)+3));
                $is_first_or_last = ($k<=1 || $k>=(ceil($last_page/$entries_per_page)-2));
                if ($is_too_long && !$is_first_or_last && !$is_near_current) {
                    /* this marks an elipsis */
                    if ($dots === false) {
                        $dots = true;
                        array_push($list_of_pages, false);
                    } else {
                        /* ignore this page */
                    }
                } else {
                    $first = ($k*$entries_per_page);
                    $last  = ((($k+1)*$entries_per_page < $last_page) ? ($k+1)*$entries_per_page : $last_page);
                    array_push($list_of_pages, array('FIRST'=>($first+1), 'LAST' => $last));
                    if ($is_near_current) {
                        $dots = false;
                    }
                }
            }
        }
        assert('is_array($list_of_pages); // Unexpected output');
        return $list_of_pages;
    }

    /**
     * smarty processor htmlPostProcessor
     *
     * @access  public
     * @static
     * @param   array   $params
     * @param   Smarty  &$smarty
     * @return  string
     */
    function htmlPostProcessor($source, &$smarty)
    {
        if (!is_string($source)) {
            return $source;
        }
        global $lcn_note, $instance_id, $YANA;

        $permission = $YANA->getVar('PERMISSION');

        /* Create field for internal form validation */
        if (!is_int($permission) || $permission < 1) {
            $source = str_replace("</form>", "<span class=\"yana_button\"><input type=\"text\" name=\"yana_url\"></span>\n</form>", $source);
        }

        /* Instance Management */
        if (isset($YANA))  {
            $lcn_key = $YANA->getVar("LICENCE.KEY");
            if (!empty($lcn_key)) {
                $a = $YANA->getVar("LICENCE");
                $lcn_note = Layout::fillTemplate($lcn_note, $a);
                $source = preg_replace("/<\/body>/i", $lcn_note."</body>", $source);
            } else {
                $source = preg_replace("/<\/body>/i", $instance_id."</body>", $source);
            }
        } else {
            $source = preg_replace("/<\/body>/i", $instance_id."</body>", $source);
        }

        return $source;
    }

    /**
     * smarty processor processLanguageInstructions
     *
     * @access  public
     * @static
     * @param   array   $params
     * @param   Smarty  &$smarty
     * @return  string
     */
    function processLanguageInstructions($source, &$smarty)
    {
        $source = preg_replace("/".DisplayUtility::_regExpToken("(\\\$LANGUAGE\..*)")."/Us", YANA_LEFT_DELIMITER."\\1|embeddedTags".YANA_RIGHT_DELIMITER, $source);
        return $source;
    }

    /**
     * smarty processor htmlPreProcessor
     *
     * @access public
     * @static
     * @param string $source
     * @param smarty &$templateClass
     * @return string
     */
    function htmlPreProcessor($source, &$templateClass)
    {
        if (!is_string($source)) {
            return $source;
        }
        global $YANA;

        /* funcionality of 'include' function has been moved to 'import', so ignore the original */
        $source = preg_replace("/".DisplayUtility::_regExpToken("include .*")."/Usi", "", $source);
        /* deactivate 'include_php' for security reasons */
        $source = preg_replace("/".DisplayUtility::_regExpToken("include_php .*")."/Usi", "", $source);
        /* deactivate 'php' for security reasons */
        $source = preg_replace("/".DisplayUtility::_regExpToken("php").".*".DisplayUtility::_regExpToken("\/php")."/Usi", "", $source);
        /* reformat abbreviated %FOO% tags to [%$FOO%] */
        $source = preg_replace("/".toRegExp(YANA_UNI_DELIMITER)."(([A-Z]|\d|_|-|\.)+)".toRegExp(YANA_UNI_DELIMITER)."/", DisplayUtility::_varToken("\\1"), $source);

        $basedir = $templateClass->get_template_vars('BASEDIR');
        settype($basedir, "string");
        $is_include = $templateClass->get_template_vars('FILE_IS_INCLUDE');
        if ($is_include) {
            $source = preg_replace("/^.*<body(.*)>(.*)<\/body>.*$/Usi", "<div\\1>\\2</div>", $source);
        }

        if (!empty($basedir)) {
            preg_match_all('/('.DisplayUtility::_regExpToken(')import\s+(?:preparser\s+|)file="(\S*)(".*').')/Ui', $source, $match2);
            for ($i=0;$i<count($match2[0]);$i++)
            {
                if (preg_match('/\sliteral\s/i' ,$match2[0][$i])) {
                    $source = preg_replace('/'.toRegExp($match2[0][$i]).'/i', preg_replace("/\sliteral\s/i", " ", $match2[0][$i]), $source);
                } else if (preg_match('/\spreparser\s/i' ,$match2[0][$i])) {
                    $source = preg_replace('/'.toRegExp($match2[0][$i]).'/i', preg_replace("/.*<body[^>]*>(.*)<\/body>.*/si", "\\1", implode("", file($basedir.$match2[2][$i]))), $source);
                } else {
                    $source = str_replace($match2[0][$i], $match2[1][$i].'import file="'.$basedir.$match2[2][$i].$match2[3][$i], $source);
                }
            }

            preg_match_all('/('.DisplayUtility::_regExpToken(')insert\s+file="(\S*)(".*').')/Ui', $source, $match2);
            for ($i=0;$i<count($match2[0]);$i++)
            {
                    $source = preg_replace('/'.toRegExp($match2[0][$i]).'/i', $match2[1][$i].'insert file="'.$basedir.$match2[2][$i].$match2[3][$i], $source);
            }

            preg_match_all('/ background\s*=\s*"(\S*)"/i', $source, $match2);
            for ($i=0;$i<count($match2[1]);$i++)
            {
                if (!preg_match('/^https?:\/\/\S*/i', $match2[1][$i]) && !preg_match('/^'.DisplayUtility::_regExpVarToken('PHP_SELF').'/i', $match2[1][$i])) {
                    $source = preg_replace('/ background\s*=\s*"'.toRegExp($match2[1][$i]).'"/i', ' background="'.$basedir.$match2[1][$i].'"', $source);
                }
            }
            preg_match_all('/ src\s*=\s*"(\S*)"/i', $source, $match2);
            for ($i=0;$i<count($match2[1]);$i++)
            {
                if (!preg_match('/ src\s?=\s?"\S*"/i', $match2[1][$i]) && !preg_match('/^https?:\/\/\S*/i', $match2[1][$i]) && !preg_match('/^'.DisplayUtility::_regExpVarToken('PHP_SELF').'/i', $match2[1][$i])) {
                    $source = preg_replace('/ src\s*=\s*"'.toRegExp($match2[1][$i]).'"/i', ' src="'.$basedir.$match2[1][$i].'"', $source);
                }
            }
            preg_match_all("/\.src\s*=\s*'(\S*)'/i", $source, $match2);
            for ($i=0;$i<count($match2[1]);$i++)
            {
                if (!preg_match("/\.src\s*=\s*'\S*'/i", $match2[1][$i]) && !preg_match('/^https?:\/\/\S*/i', $match2[1][$i]) && !preg_match('/^'.DisplayUtility::_regExpVarToken('PHP_SELF').'/i', $match2[1][$i])) {
                    $source = preg_replace("/\.src\s?=\s?'".toRegExp($match2[1][$i])."'/i", ".src='".$basedir.$match2[1][$i]."'", $source);
                }
            }
            preg_match_all("/ url\(\"([^\"]*)\"\)/i", $source, $match2);
            for ($i=0;$i<count($match2[1]);$i++)
            {
                if (!preg_match("/ url\(\"[^\"]*\"\)/i", $match2[1][$i]) && !preg_match('/^https?:\/\/\S*/i', $match2[1][$i]) && !preg_match('/^'.DisplayUtility::_regExpVarToken('PHP_SELF').'/i', $match2[1][$i])) {
                    $source = preg_replace("/ url\(\"".toRegExp($match2[1][$i])."\"\)/i", " url(\"".$basedir.$match2[1][$i]."\")", $source);
                }
            }
            preg_match_all("/ url\('([^']*)'\)/i", $source, $match2);
            for ($i=0;$i<count($match2[1]);$i++)
            {
                if (!preg_match("/ url\('[^']*'\)/i", $match2[1][$i]) && !preg_match('/^https?:\/\/\S*/i', $match2[1][$i]) && !preg_match('/^'.DisplayUtility::_regExpVarToken('PHP_SELF').'/i', $match2[1][$i])) {
                    $source = preg_replace("/ url\('".toRegExp($match2[1][$i])."'\)/i", " url('".$basedir.$match2[1][$i]."')", $source);
                }
            }
            preg_match_all('/ href\s*=\s*"([^"]*)"/i', $source, $match2);
            for ($i=0;$i<count($match2[1]);$i++)
            {
                if (!preg_match('/ href\s*=\s*"[^"]*"/i', $match2[1][$i]) && !preg_match('/^javascript:/i', $match2[1][$i]) && !preg_match('/^\&\#109\;\&\#97\;\&\#105\;\&\#108\;\&\#116\;\&\#111\;\&\#58\;\S*/i', $match2[1][$i]) && !preg_match('/^mailto:\S*/i', $match2[1][$i]) && !preg_match('/^https?:\/\/\S*/i', $match2[1][$i]) && !preg_match('/^'.DisplayUtility::_regExpVarToken('PHP_SELF').'/i', $match2[1][$i])) {
                    $source = preg_replace('/ href\s?=\s?"'.toRegExp($match2[1][$i]).'"/i', ' href="'.$basedir.$match2[1][$i].'"', $source);
                }
            }

        }

        return DisplayUtility::scanForAt($source);
    }

    /**
     * smarty outputfilter outputfilter
     *
     * @access  public
     * @static
     * @param   string  $source
     * @param   Smarty  &$templateClass
     * @return  string
     */
    function outputFilter($source, &$templateClass)
    {
        /* Create form id */
        global $YANA;
        if ($YANA->getVar('DISABLE_FORM_ID') !== true) {
            $yana_form_id = md5(uniqid(rand(), true));
            if (strpos($source, "</form>") !== false) {
                /* insert form id */
                $source   = str_replace("</form>", "<span class=\"yana_button\"><input type=\"text\" name=\"yana_form_id\" value=\"$yana_form_id\"></span>\n</form>", $source);
                $_SESSION['yana_form_id'] = $yana_form_id;
                $YANA->setVar('DISABLE_FORM_ID', true);
            }
        }

        $basedir = $templateClass->get_template_vars('BASEDIR');

        /* add meta data to header */
        if (strpos($source, '</head>') > -1) {

            /* import stylesheets */
            $style_list = $templateClass->get_template_vars('SYSTEM_STYLE');
            assert('is_null($style_list) || is_array($style_list);');
            if (is_array($style_list) && count($style_list) > 0) {
                $style_list = array_reverse($style_list, true);
                assert('!isset($stylesheet); /* cannot redeclare variable $stylesheet */');
                foreach ($style_list as $stylesheet)
                {
                    $source = preg_replace('/^(\s*<head[^>]*>)/m', '${1}'."\n".'        '.DisplayUtility::css($stylesheet), $source);
                }
                unset($stylesheet);
            }
    
            /* import javascripts */
            $script_list = $templateClass->get_template_vars('SYSTEM_SCRIPT');
    
            assert('is_null($script_list) || is_array($script_list);');
            if (is_array($script_list) && count($script_list) > 0) {
                assert('!isset($script); /* cannot redeclare variable $script */');
                foreach ($script_list as $script)
                {
                    $source = preg_replace('/^(\s*<\/head>)/m', '        '.DisplayUtility::script($script)."\n".'${1}', $source);
                }
                unset($script);
            }
            
            /* import microsummaries */
            $summary_list = $templateClass->get_template_vars('MICROSUMMARY');

            assert('is_null($summary_list) || is_array($summary_list);');
            if (is_array($summary_list) && count($summary_list) > 0) {
                assert('!isset($summary); /* cannot redeclare variable $summary */');
                foreach ($summary_list as $summary)
                {
                    $source = preg_replace('/^(\s*<\/head>)/m', '        '.DisplayUtility::microsummary($summary)."\n".'${1}', $source);
                }
                unset($summary);
            }

        } /* end if */

        return $source;
    }

    /**
     * smarty modifier date
     *
     * @access  public
     * @static
     * @param   string  $time
     * @return  string
     */
    function date($time)
    {
        settype($time, "integer");
        global $YANA;

        if (empty($time)) {
            $time = time();
        }
        
        $timeformat = $YANA->getVar("DATE.".$YANA->getVar("PROFILE.TIMEFORMAT"));

        $script = "";
        /* provide javascript */
        if ((is_array($timeformat) && isset($timeformat['JS'])) || is_string($timeformat)) {
            $script .= '<script type="text/javascript" language="JavaScript">';
            $script .= 'date=new Date('.$time."000);document.write(".((is_string($timeformat)) ? $timeformat : $timeformat['JS']).");</script>";
        }
        /* provide textual representation for fall back */
        if (is_array($timeformat) && isset($timeformat['PHP'])) {
            $script .= '<noscript>'.date($timeformat['PHP'],$time).'</noscript>';
        }
        return $script;

    }

    /**
     * smarty modifier entities
     *
     * @access  public
     * @static
     * @param   string  $string
     * @return  string
     */
    function entities($string)
    {
        return htmlentities($string);
    }

    /**
     * smarty modifier url
     *
     * @access  public
     * @static
     * @param   string   $string           url parameter list
     * @param   boolean  $asString         decide wether entities in string should be encoded or not
     * @param   boolean  $asAbsolutePath   decide wether function should return relative or absolut path
     * @return  string
     */
    function url($string, $asString = false, $asAbsolutePath = true)
    {

        settype($string, "string");
        global $YANA;

        preg_match_all("/(&|^)(.*)=(.*)(&|$)/U", $string, $m);
        for ($i=0;$i<count($m[0]);$i++)
        {
            $string = preg_replace("/".toRegExp($m[0][$i])."/", $m[1][$i].urlencode($m[2][$i])."=".urlencode($m[3][$i]).$m[4][$i], $string);
        }

        $url = "";

        if ($asAbsolutePath === true) {
            if (isset($_SERVER['HTTPS']) && strcasecmp($_SERVER['HTTPS'], "on") === 0) {
                $url = "https://";
            } else {
                $url = "http://";
            }
            if (isset($_SERVER['HTTP_HOST'])) {
                $url .= $_SERVER['HTTP_HOST'];
            } else {
                $url .= $_SERVER['SERVER_NAME'];
            }
            $url .= dirname($_SERVER['PHP_SELF']). '/';
        }
        if (isset($YANA)) {
            $url .= $YANA->getVar('PHP_SELF')."?".((@session_id() != "") ? @session_name()."=".@session_id() : "");
            if ($asString === true) {
                $url .= "&id=".$YANA->getVar('ID')."&".$string;
            } else {
                $url .= "&amp;id=".$YANA->getVar('ID')."&amp;".htmlentities($string);
            }
        } else {
            $url .= $_SERVER['PHP_SELF']."?".((@session_id() != "") ? @session_name()."=".@session_id() : "");
        }
        return $url;

    }

    /**
     * smarty modifier href
     *
     * @access  public
     * @static
     * @param   string  $time
     * @return  string
     */
    function href($string)
    {

        return '"'.DisplayUtility::url($string, false, false).'"';

    }

    /**
     * smarty modifier microsummary
     *
     * @access  public
     * @static
     * @param   string  $target
     * @return  string
     */
    function microsummary($target)
    {
        settype($url, "string");
        if (!preg_match("/^\w[\w\d-_]*$/si", $target)) {
            return "";
        } else {
            return '<link rel="microsummary" href="'.$_SERVER['PHP_SELF']."?action=get_microsummary&amp;target=$target".'" type="text/plain">';
        }
    }

    /**
     * smarty modifier css
     *
     * @access  public
     * @static
     * @param   string  $url
     * @return  string
     */
    function css($url)
    {
        settype($url, "string");
        if (!preg_match("/^[\w-_\.\/]+\.css$/si", $url)) {
            return "";
        } else {
            return '<link rel="stylesheet" type="text/css" href="'.$url.'">';
        }
    }

    /**
     * smarty modifier script
     *
     * @access  public
     * @static
     * @param   string  $time
     * @return  string
     */
    function script($string)
    {
        settype($string, "string");
        if (!preg_match("/^[\w-_\.\/]+\.js$/si", $string)) {
            return "";
        } else {
            return '<script type="text/javascript" language="javascript" src="'.$string.'"></script>';
        }
    }

    /**
     * smarty modifier embeddedTags
     *
     * @access  public
     * @static
     * @param   string  $source
     * @param   bool    $use_html_entities
     * @return  string
     */
    function embeddedTags($txt, $use_html_entities = true)
    {
        if (!is_string($txt)) {
            return $txt;
        }

        if (!is_bool($use_html_entities)) {
            trigger_error("Wrong argument type for argument 2 in function ".__FUNCTION__."(). Boolean expected, found '".gettype($input)."' instead.", E_USER_ERROR);
            return $txt;
        }

        /* if not necessary -> skip the whole section for better performance */
        if (is_int(strpos($txt, '['))) {

            $invalid_resource = '<img title="the resource %RESOURCE% was blocked because it contained illegal characters" alt="invalid %RESOURCE%" border="0" src="%DATADIR%icon_x.gif">';

            $txt = preg_replace("/\[php\](.*)(\[\/php\]|$)/Usi", "[code]<?php\n\\1\n?>[/code]", $txt);
            while (preg_match('/\[code\](.*)(\[\/code\]|$)/Us', $txt, $m))
            {
                $m[1] = preg_replace("/\[\/?(b|i|emp|h|code|php|img|mark|color|url|mail)\]/si", "", $m[1]);
                /* This is to avoid double quoting , since highlight_string() will quote the string again. */
                $m[1] = html_entity_decode($m[1]);
                $m[1] = '<code style="white-space: pre-wrap">'.highlight_string($m[1],true).'</code>';
                $txt  = str_replace($m[0], $m[1], $txt);
            }
            unset($m);
            $txt = preg_replace("/\[b\](.*)(\[\/b\]|$)/Us",               "<span style=\"font-weight: bold;\">\\1</span>", $txt);
            $txt = preg_replace("/\[i\](.*)(\[\/i\]|$)/Us",               "<span style=\"font-style: italic;\">\\1</span>", $txt);
            $txt = preg_replace("/\[u\](.*)(\[\/u\]|$)/Us",               "<span style=\"text-decoration: underline;\">\\1</span>", $txt);
            $txt = preg_replace("/\[emp\](.*)(\[\/emp\]|$)/Us",           "<span class=\"EMPHASIZE\">\\1</span>", $txt);
            $txt = preg_replace("/\[h\](.*)(\[\/h\]|$)/Us",               "<p class=\"HIGHLIGHT\">\\1</p>", $txt);
            $txt = preg_replace("/\[c\](.*)(\[\/c\]|$)/Us",               "<div class=\"KOMMENTAR\">\\1</div>", $txt);
            $txt = preg_replace("/\[hide\](.*)(\[\/hide\]|$)/Us",         "<div style=\"padding: 5px; border: 1px solid #555555; background-color: #CCCCCC; color: #CCCCCC\" onMouseOver=\"this.style.backgroundColor='#F0F0F0'; this.style.color='#333333'\" onMouseOut=\"this.style.backgroundColor='#CCCCCC'; this.style.color='#CCCCCC'\">\\1</div>", $txt);

            $txt = preg_replace("/\[small\](.*)(\[\/small\]|$)/Us",       "<span style=\"font-size: x-small\">\\1</span>", $txt);
            $txt = preg_replace("/\[big\](.*)(\[\/big\]|$)/Us",           "<span style=\"font-size: large\">\\1</span>", $txt);
            $txt = preg_replace("/\[mark\](.*)(\[\/mark\]|$)/Usi",         "<span style=\"background-color:#FFFF00\">\\1</span>", $txt);
            $txt = preg_replace("/\[mark=(\w+|\#[\da-fA-F]{3}|\#[\da-fA-F]{6})\](.*)(\[\/mark\]|$)/Us", '<span style="background-color:${1}">${2}</span>', $txt);
            $txt = preg_replace("/\[color\](.*)(\[\/color\]|$)/Us",       "<span style=\"color:#000000\">\\1</span>", $txt);
            $txt = preg_replace("/\[color=(\w+|\#[\da-fA-F]{3}|\#[\da-fA-F]{6})\](.*)(\[\/color\]|$)/Us", '<span style="color:${1}">${2}</span>', $txt);

            $txt = preg_replace("/\[mail=mailto:(.*)\]/Us",               "[mail=\\1]", $txt);
            $txt = preg_replace("/\[mail]mailto:(.*)\[\/mail\]/Us",       "[mail]\\1[/mail]", $txt);

            /* images */
            $matches = array();
            while (preg_match("/\[img\](.*)\[\/img\]/U", $txt, $matches))
            {
                if (preg_match("/^[\w\d-_\/]+\.(png|jpg|gif|jpeg)$/i", $matches[1], $ext)) {
                    $txt = str_replace($matches[0], '<img alt="" border="0" src="'.htmlentities(strip_tags(preg_replace("/\[wbr\]/i", "", $matches[1]))).'" style="max-width: 320px; max-height: 240px" onload="javascript:if(this.width>320) { this.width=320; }; if(this.height>240) { this.height=240; };">', $txt);
                } else {
                    $txt = str_replace($matches[0], str_replace('%RESOURCE%', 'image', $invalid_resource), $txt);
                }
            } /* end foreach */
            $matches = array();

            /* mail */
            /* may contain word-/line- breaks that need to be removed */
            while (preg_match('/\[mail=[^\[\]]*\[wbr\]/i', $txt))
            {
                $txt   = preg_replace('/(\[mail=[^\[\]]*)\[wbr\]/i', '${1}', $txt);
            }
            unset($count);
            while (preg_match("/\[mail=(.*)\](.*)\[\/mail\]/Usi", $txt, $matches1) || preg_match("/\[mail\](.*)\[\/mail\]/Ui", $txt, $matches2))
            {
                if (!empty($matches1)) {
                    $mail_match =& $matches1[0];
                    $mail_href  =  $matches1[1];
                    $mail_text  =  $matches1[2];
                } else if (!empty($matches2)) {
                    $mail_match =& $matches2[0];
                    $mail_href  =  $matches2[1];
                    $mail_text  =  $matches2[1];
                    $mail_text  =  preg_replace('/ ?(\[wbr\]|\[br\])/','',$mail_text);
                    if (strlen($mail_text) > 40) {
                        $mail_text = substr($mail_text,0,34) . "..." . substr($mail_text,-3);
                    }
                }
                /* Count in the possibility of some nasty guy injecting white space characters */
                $mail_href = preg_replace('/[\x00-\x1f]/','',$mail_href);
                /* Although there should not be any tags - you never know ... */
                $mail_href = strip_tags($mail_href);
                $mail_href = preg_replace('/ ?(\[wbr\]|\[br\])/','',$mail_href);
                $mail_href = preg_replace('/^mailto:/i', '', $mail_href);
                $mail_href = untaintInput($mail_href, 'mail');
                $mail_href = htmlentities($mail_href);
                if (!empty($mail_href)) {
                    $txt = str_replace($mail_match, '<a href="&#109;&#97;&#105;&#108;&#116;&#111;&#58;'.$mail_href.'" target="_blank">'.$mail_text.'</a>', $txt);
                } else {
                    $txt = str_replace($mail_match, str_replace('%RESOURCE%', 'mail', $invalid_resource), $txt);
                }
            } /* end while */
            $matches1 = array();
            $matches2 = array();

            /* url */
            /* may contain word-/line- breaks that need to be removed */
            while (preg_match('/\[url=[^\[\]]*(?:\[wbr\]|\[br\])/si', $txt))
            {
                $txt   = preg_replace('/(\[url=[^\[\]]*)(?:\[wbr\]|\[br\])/si', '${1}', $txt);
            }
            unset($count);
            while (preg_match("/\[url=(.*)\](.*)\[\/url\]/Usi", $txt, $matches1) || preg_match("/\[url\](.*)\[\/url\]/Ui", $txt, $matches2))
            {
                if (!empty($matches1)) {
                    $uri_match =& $matches1[0];
                    $uri_href  =  $matches1[1];
                    $uri_text  =  $matches1[2];
                } else if (!empty($matches2)) {
                    $uri_match =& $matches2[0];
                    $uri_href  =  $matches2[1];
                    $uri_text  =  $matches2[1];
                    $uri_text  =  preg_replace('/ ?(\[wbr\])/','',$uri_text);
                    if (strlen($uri_text) > 40) {
                        $uri_text = substr($uri_text,0,34) . "..." . substr($uri_text,-3);
                    }
                }
                /* Count in the possibility of some nasty guy injecting white space characters */
                $uri_href = preg_replace('/[\x00-\x1f]/','',$uri_href);
                /* Although there should not be any tags - you never know ... */
                $uri_href = strip_tags($uri_href);
                $uri_href = preg_replace('/ ?(\[wbr\]|\[br\])/','',$uri_href);
                $uri_href = untaintInput($uri_href, 'url');
                $uri_href = htmlentities($uri_href);
                if (!preg_match('/^(https?:\/\/|ftp:\/\/)/', $uri_href) && preg_match('/^[^:]+:/', $uri_href)) {
                    $uri_href = '';
                } else if (!preg_match('/^[^:]+:/', $uri_href)) {
                    $uri_href = 'http://'.$uri_href;
                }
                if (!empty($uri_href)) {
                    $txt = str_replace($uri_match, '<a href="'.$uri_href.'" target="_blank">'.$uri_text.'</a>', $txt);
                } else {
                    $txt = str_replace($uri_match, str_replace('%RESOURCE%', 'uri', $invalid_resource), $txt);
                }
                unset($uri_match);
                unset($uri_href);
                unset($uri_text);
            }
            $matches1 = array();
            $matches2 = array();

            /* line break */
            $txt = str_replace('[br]', '<br>', $txt);

            /* word break */
            $txt = str_replace('[wbr]', '<span style="font-size: 0px"> </span>', $txt);

            /* remove uncaught tags */
            $txt = preg_replace("/\[\/?(b|i|emp|h|code|php|img|mark|color|url|mail)\]/si", "", $txt);
        }

        assert('is_string($txt);');
        return $txt;
    }

    /**
     * smarty modifier scanForAt
     *
     * @access  public
     * @static
     * @param   string  $source
     * @return  string
     */
    function scanForAt($source)
    {

        if (!is_string($source)) {
            return $source;
        } else {
            preg_match_all("/[\w\.\-_]+@[\w\.\-_]+/", $source, $matches);

            for ($i=0; $i<count($matches[0]); $i++)
            {
                $source = str_replace($matches[0][$i], String::htmlEntities($matches[0][$i]), $source);
            }

            return $source;
        }

    }

    /**
     * smarty modifier smilies
     *
     * @access  public
     * @static
     * @param   string  $txt
     * @return  string
     */
    function smilies($txt)
    {
        assert('empty($txt) || is_string($txt); // Wrong argument type. String expected found '.gettype($txt).' instead.');
        if (!is_string($txt)) {
            return $txt;
        }

        global $YANA;
        if (isset($YANA)) {
            $smilies = $YANA->getVar('SMILIES');
            $smilies_dir = $YANA->getVar('PROFILE.SMILEYDIR');
        } else {
            global $smilies, $smilies_dir;
        }

        /* default smilies */
        if (isset($smilies) && is_array($smilies)) {
            if (in_array("smile", $smilies))        {
                $txt = preg_replace("/:[\-oO]?\)/", '<img alt=":-)" border="0" hspace="2" src="'.$smilies_dir.'smile.gif">', $txt);
            }
            if (in_array("teufelchen", $smilies))   {
                $txt = preg_replace("/>:[\-oO]?\)/", '<img alt="&gt-)" border="0" hspace="2" src="'.$smilies_dir.'teufelchen.gif">', $txt);
            }
            if (in_array("boese", $smilies))        {
                $txt = preg_replace("/:[\-oO]?\(/", '<img alt=":-(" border="0" hspace="2" src="'.$smilies_dir.'boese.gif">', $txt);
            }
            if (in_array("keinemeinung", $smilies)) {
                $txt = preg_replace("/:[\-oO]?\|/", '<img alt=":-|" border="0" hspace="2" src="'.$smilies_dir.'keinemeinung.gif">', $txt);
            }
            if (in_array("lachen", $smilies))       {
                $txt = preg_replace("/:[\-oO]?D/", '<img alt=":-D" border="0" hspace="2" src="'.$smilies_dir.'lachen.gif">', $txt);
            }
            if (in_array("grinsen", $smilies))      {
                $txt = preg_replace("/;[\-oO]?\)/", '<img alt=";-)" border="0" hspace="2" src="'.$smilies_dir.'grinsen.gif">', $txt);
            }
            if (in_array("zunge", $smilies))        {
                $txt = preg_replace("/:[\-oO]?P/", '<img alt=":-P" border="0" hspace="2" src="'.$smilies_dir.'zunge.gif">', $txt);
            }
            if (in_array("traurig", $smilies))      {
                $txt = preg_replace("/:[\'\*]\(/", '<img alt=":*(" border="0" hspace="2" src="'.$smilies_dir.'traurig.gif">', $txt);
            }
        }

        /* if not necessary -> skip the whole section for better performance */
        if (is_int(strpos($txt, ':'))) {
            /* Emot-Codes */
            for ($j=0;$j<count($smilies);$j++)
            {
                while (preg_match("/:".$smilies[$j].":(\s|\[wbr\]|\[br\]|<br \/>)*:".$smilies[$j].":/i", $txt))
                {
                    $txt = preg_replace("/:".$smilies[$j].":(\s|\[wbr\]|\[br\]|<br \/>)*:".$smilies[$j].":/i", ':'.$smilies[$j].':', $txt);
                }
                $txt = preg_replace("/:".addcslashes($smilies[$j], "+()[]{}.?*/\\$^").":/", '<img alt="" border="0" hspace="2" src="'.$smilies_dir.$smilies[$j].'.gif">', $txt);
            }
        }

        assert('is_string($txt);');
        return $txt;
    }

    /**
     * smarty function printArray
     *
     * @access  public
     * @static
     * @param   array  $array
     * @return  string
     */
    function printArray($params, &$smarty)
    {
        if (!isset($params['value'])) {
            return "";
        } else {
            $array = $params['value'];
        }
        if (is_string($array)) {
            $array = SML::decode($array);
        }
        if (is_array($array)) {
            return '<pre>'.htmlentities(SML::encode($array)).'</pre>';
        } else {
            return $array;
        }
    }

    /**
     * smarty function guiEmbeddedTags
     *
     * @static
     * @access  public
     * @name    DisplayUtility::guiEmbeddedTags()
     * @param   array  $params
     * @param   smarty &$smarty
     * @return  string
     *
     * @see     DisplayUtility::loadSmilies()
     */
    function guiEmbeddedTags($params, &$smarty)
    {
        global $YANA;

        $list_of_tags = array('b','i','u','h','emp','c','small','big','hide','code','img','url','mail','color','mark');

        /* Argument 'target' */
        if (!isset($params['target'])) {
            $smarty->trigger_error("Missing argument 'target' in function ".__FUNCTION__."().", E_USER_WARNING);
            return "";
        } else if (!is_string($params['target'])) {
            $smarty->trigger_error("Wrong argument type for argument 'target' in function ".__FUNCTION__."(). String expected, found '".gettype($params['target'])."' instead.", E_USER_WARNING);
            return "";
        } else if (!preg_match('/^[\w\d_]+$/is',$params['target'])) {
            $smarty->trigger_error("Argument 'target' contains illegal characters for function ".__FUNCTION__."(). Must be a valid id.", E_USER_WARNING);
            return "";
        } else {
            $target = $params['target'];
        }

        /* Argument 'show' */
        if (isset($params['show']) && !is_string($params['show'])) {
            $smarty->trigger_error("Wrong argument type for argument 'show' in function ".__FUNCTION__."(). String expected, found '".gettype($params['show'])."' instead.", E_USER_WARNING);
            return "";
        } else if (isset($params['show']) && !preg_match('/^(\w+|\||-)(,(\w+|\||-))*$/is',$params['show'])) {
            $smarty->trigger_error("Argument 'show' contains illegal characters for function ".__FUNCTION__."().", E_USER_WARNING);
            return "";
        } else if (!isset($params['show'])) {
            $show =& $list_of_tags;
        } else {
            $show = explode(',', strtolower($params['show']));
        }

        /* Argument 'hide' */
        if (!empty($params['hide']) && !is_string($params['hide'])) {
            $smarty->trigger_error("Wrong argument type for argument 'hide' in function ".__FUNCTION__."(). String expected, found '".gettype($params['hide'])."' instead.", E_USER_WARNING);
            return "";
        } else if (!empty($params['hide']) && !preg_match('/^[\w,]+$/is',$params['hide'])) {
            $smarty->trigger_error("Argument 'hide' contains illegal characters for function ".__FUNCTION__."().", E_USER_WARNING);
            return "";
        } else if (empty($params['hide'])) {
            $hide = array();
        } else {
            $hide = explode(',', strtolower($params['hide']));
        }

        assert('is_array($show);');
        assert('is_array($hide);');

        $tags = array_diff($show, $hide);

        /* create document */
        $document = new smartTemplate();
        $document->insert('TAGS', $tags);
        $document->insert('TARGET', $target);
        $document->insert('LANGUAGE', $YANA->getVar('LANGUAGE'));
        $document->setFilename($YANA->skin->getFile("gui_embedded_tags"));

        return $document->get();
    }

    /**
     * smarty function guiSmlLoad
     *
     * The function sml_load() provides the same interface as
     * the built-in smarty function config_load().
     * The difference is, that sml_load() works on SML files.
     *
     * In addition you may use the scope 'template_vars', which
     * is unique for sml_load(). This scope merges the values
     * with the current array of template vars instead of copying
     * them to the config vars of the template. This enables you
     * to handle loaded SML structure as common vars and provides
     * access to nested structures inside SML files, which is
     * a feature that is not available for normal smarty config files.
     *
     * @static
     * @access  public
     * @name    DisplayUtility::guiSmlLoad()
     * @param   array  $params
     * @param   smarty &$smarty
     * @return  string
     */
    function guiSmlLoad($params, &$smarty)
    {

        /* input checking*/

        if (empty($params['file'])) {
            $smarty->trigger_error("Missing argument 'file' in function sml_load()", E_USER_WARNING);
            return "";
        } else if (!is_string($params['file'])) {
            $smarty->trigger_error("Wrong argument type for argument 'file' in function sml_load(). String expected, found '".gettype($params['file'])."' instead.", E_USER_WARNING);
            return "";
        } else if (!preg_match('/^[\w\d-_]+\.(?:sml|config)$/i', $params['file'])) {
            $smarty->trigger_error("Argument 'file' contains illegal characters for function sml_load().", E_USER_WARNING);
            return "";
        } else if (!is_file($smarty->config_dir.$params['file'])) {
            $smarty->trigger_error("Argument 'file' is not a valid file path in function sml_load().", E_USER_WARNING);
            return "";
        } else {
            $in_file = $smarty->config_dir.$params['file'];
        } /* end if */

        if (empty($params['section']) || $params['section'] === '*') {
            $in_section = null;
        } else if (!is_string($params['section'])) {
            $smarty->trigger_error("Wrong argument type for argument 'section' in function sml_load(). String expected, found '".gettype($params['section'])."' instead.", E_USER_WARNING);
            return "";
        } else if (!preg_match('/^([\w\d-_\.]+)$/i', $params['section'])) {
            $smarty->trigger_error("Argument 'section' contains illegal characters for function sml_load().", E_USER_WARNING);
            return "";
        } else {
            $in_section = $params['section'];
        } /* end if */

        if (empty($params['scope'])) {
            if (empty($params['global'])) {
                $in_scope = 'local';
            } else {
                if (!is_boolean($params['global'])) {
                    $smarty->trigger_error("Wrong argument type for argument 'global' in function sml_load(). Boolean expected, found '".gettype($params['global'])."' instead.", E_USER_WARNING);
                    return "";
                } else {
                    $smarty->trigger_error("Argument 'global' is deprecated in function sml_load(). You should consider using scope='parent' (equals global=true) or scope='local' (equals global=false) instead.", E_USER_NOTICE);
                    if ($params['global'] === true) {
                        $in_scope = 'parent';
                    } else if ($params['global'] === false) {
                        $in_scope = 'local';
                    } else {
                        $in_scope = 'local';
                    } /* end if */
                } /* end if */
            } /* end if */
        } else if (!is_string($params['scope'])) {
            $smarty->trigger_error("Wrong argument type for argument 'scope' in function sml_load(). String expected, found '".gettype($params['scope'])."' instead.", E_USER_WARNING);
            return "";
        } else {
            switch(strtolower($params['scope'])) {
                case 'global':
                    $in_scope = 'global';
                break;

                case 'local':
                    $in_scope = 'local';
                break;

                case 'parent':
                    $in_scope = 'parent';
                break;

                case 'template_vars':
                    $in_scope = 'template_vars';
                break;

                default:
                    $smarty->trigger_error("Argument 'scope' has an illegal value for function sml_load(). Accepted values are: 'template_vars', 'global', 'local', 'parent'.", E_USER_WARNING);
                    return "";
                break;
            } /* end switch */
        } /* end if */

        unset($params);

        /* cache handling */
        $cacheFile = $smarty->compile_dir.'sml_load'.md5($in_file).'.php';
        if ($smarty->force_compile == true || !file_exists($cacheFile) || filemtime($in_file) > filemtime($cacheFile)) {
            $forceCompile = true;
        } else {
            $forceCompile = false;
        }
        clearstatcache();

        /* business logic */
        if ($forceCompile === true) {
            assert('!isset($array);');
            $array = SML::getFile($in_file, CASE_UPPER);
            $cache = new SecureFileStream($cacheFile);
            $cache->read();
            if (!$cache->exists()) {
                $cache->create();
            }
            $cache->insert('<?php $array = unserialize(\''.serialize($array).'\'); ?>');
            $cache->write();
            unset($cache);
        } else {
            include($cacheFile);
        }
        if (!empty($in_section)) {
            $array = Hashtable::get($array, $in_section);
        }

        /* resolve scope argument */
        if ($in_scope == 'template_vars') {
            $i_max = -1;
        } else if ($in_scope == 'local') {
            $i_max = 0;
        } else if ($in_scope == 'parent') {
            $i_max = 1;
        } else if ($in_scope == 'global') {
            $i_max = count($smarty->_config);
        } else {
            $i_max = UNDEFINED;
            $smarty->_config_overwrite(null);
            return "";
        }

        /* copy values to smarty config vars */
        if ($in_scope == 'template_vars') {
            $keys = array_keys($array);
            for ($i=0;$i<count($keys);$i++)
            {
                $smarty->assign_by_ref($keys[$i], $array[$keys[$i]], true);
            }
        } else {
            for ($i = 0; $i <= $i_max; $i++)
            {
                if (!isset($smarty->_config[$i])) {
                    $smarty->_config[$i] = array('vars' => array(), 'files' => array());
                }
                if (empty($smarty->_config[$i]['vars'])) {
                    $smarty->_config[$i]['vars'] = $array;
                } else {
                    $smarty->_config[$i]['vars'] = mergeArrays($smarty->_config[$i]['vars'], $array);
                }
                $smarty->_config[$i]['files'][$in_file] = true;

            }
        }

        return "";
    }

    /**
     * smarty function guiSmilies
     *
     * @static
     * @access  public
     * @name    DisplayUtility::guiSmilies()
     * @param   array  $params
     * @param   smarty &$smarty
     * @return  string
     *
     * @see     DisplayUtility::loadSmilies()
     */
    function guiSmilies($params, &$smarty)
    {
        global $YANA;
        $TABLE_smilies = '<table summary="smilies" class="gui_generator_smilies"><tr>';
        $title_smilies = $YANA->getVar("LANGUAGE.TITLE_SMILIES");
        $smileyDir     = $YANA->getVar("PROFILE.SMILEYDIR");
        $smilies       = $YANA->getVar("SMILIES");
        settype($params['width'], "integer");
        if ($params['width'] < 1) {
            $params['width'] = 1;
        }

        for ($j=0;$j<count($smilies);$j++)
        {
            if ($j % $params['width'] == 0 && $j > 0) {
                $TABLE_smilies .= '</tr><tr>';
            }
            $TABLE_smilies .= '<td><a title="'.$title_smilies.'" href="javascript://:'.urlencode($smilies[$j]).':"><img alt="'.$smilies[$j].'" src="'.$smileyDir.$smilies[$j].'.gif" onmousedown="addIcon(\':'.$smilies[$j].':\',\''.$params['target'].'\',event)"></a></td>'."\n";
        }

        return $TABLE_smilies."</tr></table>";
    }

    /**
     * loadSmilies
     *
     * @static
     * @access  public
     * @name    DisplayUtility::loadSmilies()
     * @param   string  $user_dir (optional)
     *
     * @see     DisplayUtility::guiSmilies()
     */
    function loadSmilies($user_dir="./")
    {
        assert('is_dir($user_dir); /* The value \''.$user_dir.'\' is not a directory */');
        if (!is_string($user_dir)) {
            trigger_error("Wrong type for argument 1. String expected found '".gettype($user_dir)."' instead.",E_USER_WARNING);
        } else {
            global $YANA;
            if (isset($YANA)) {
                $smilies_dir = $YANA->getVar('PROFILE.SMILEYDIR');
            } else {
                global $smilies, $smilies_dir;
                $smilies_dir = $user_dir;
            }
    
            if (is_dir($smilies_dir)) {
                if (empty($smilies) || is_null($smilies)) {
                    $smilies = array();
        
                    $dir = dir($smilies_dir);
                    while ($txt = $dir -> read())
                    {
                        if (preg_match("/\.gif$/i", $txt)) {
                            $smilies[count($smilies)] = substr($txt, 0, strrpos($txt, "."));
                        }
                    }
                    $dir->close();
                    sort($smilies);
                }
        
                if (isset($YANA)) {
                    $YANA->setVar('SMILIES', $smilies);
                }
            } else {
                trigger_error("Unable to load smilies. The directory '${smilies_dir}' does not exist.",E_USER_WARNING);
            }
        } /* end if */
    }

    /**
     * smarty function guiImport
     *
     * @static
     * @access  public
     * @param   string  $time
     * @param   smarty  &$smarty
     * @return  string
     */
    function guiImport($param, &$smarty)
    {
        $document = new smartTemplate($param['file']);
        unset($param['file']);
        if (count($param)>0) {
            $document->insertByReference('*',$param);
        }
        $document->insert('FILE_IS_INCLUDE', "true");
        $include = $document->get();
        return $include;
    }

    /**
     * create a token with quoted delimiters
     *
     * @static
     * @access  private
     * @param   string  $input
     * @return  string
     */
    function _regExpToken($input)
    {
        assert('is_string($input);');
        settype($input, "string");
        return preg_quote(YANA_LEFT_DELIMITER).$input.preg_quote(YANA_RIGHT_DELIMITER);
    }

    /**
     * create a token refering to a variable with quoted delimiters
     *
     * @static
     * @access  private
     * @param   string  $input
     * @return  string
     */
    function _regExpVarToken($input)
    {
        assert('is_string($input);');
        settype($input, "string");
        return preg_quote(YANA_LEFT_DELIMITER).'\$'.strtoupper($input).preg_quote(YANA_RIGHT_DELIMITER);
    }

    /**
     * create a token refering to a variable
     *
     * @static
     * @access  private
     * @param   string  $input
     * @return  string
     */
    function _varToken($input)
    {
        assert('is_string($input);');
        settype($input, "string");
        return YANA_LEFT_DELIMITER.'$'.strtoupper($input).YANA_RIGHT_DELIMITER;
    }

    /**
     * create a token
     *
     * @static
     * @access  private
     * @param   string  $input
     * @return  string
     */
    function _token($input)
    {
        assert('is_string($input);');
        settype($input, "string");
        return YANA_LEFT_DELIMITER.strtoupper($input).YANA_RIGHT_DELIMITER;
    }

}
?>