<?php
/**
 * YANA library
 *
 * @package yana
 * @license http://creativecommons.org/licenses/by-nc-sa/2.5/
 */

/**
 * simulate a database
 *
 * this class simulates a sql-database on a flat-file
 *
 * @access      public
 * @package     yana
 * @subpackage  io
 */
class FileDb extends BufferedDbStream
{

    /**
     * constructor
     *
     * Create a new instance of this class.
     *
     * @param string $filename (optional)
     */
    function FileDb($filename = "")
    {
        settype($filename, "string");

        $this->database = new FileDb_connection($filename);
        $this->database->autoCommit(false);
        $this->filename = $filename;

        if (empty($this->database)) {
            /* connection failed - abort */
            $this->exists = YANA_DATABASE_FAILURE;
        } else if (empty($filename)) {
            /* no structure file available, estimate structure */
            $this->exists = YANA_DATABASE_NO_FILE;
        } else {
            $this->structure = new StructureFile($this->filename);
            $test = $this->structure->init();
            if ($test === false) {
                /* given file does not exist */
                unset($this->structure);
                $this->exists = YANA_DATABASE_NOT_AVAILABLE;
                trigger_error("Unable to initialize database. Structure file '${filename}' not found.", E_USER_WARNING);
            } else {
                /* all fine, proceed */
                $this->exists = YANA_DATABASE_OK;
                $this->database->setStructure($this->structure);
            }
        }
    }

    /**
     * not implemented query
     *
     * @access public
     * @param string $query
     * @return mixed
     * @ignore
     */
    function query($query)
    {
        /* intentionally left blank */
        return null;
    }

    /**
     * not implemented importSQL
     *
     * file database is created automatically on first use - nothing to do here!
     *
     * @access  public
     * @name    FileDb::importSQL()
     * @param   string  $sqlFile
     * @return  bool
     *
     * @ignore
     */
    function importSQL($sqlFile)
    {
        /* intentionally left blank */
        return true;
    }

    /**
     * not implemented buildStructure
     *
     * @access public
     * @ignore
     */
    function buildStructure()
    {
        /* intentionally left blank */
    }

    /**
     * isError
     *
     * @access private
     * @param mixed $result
     * @return boolean
     * @ignore
     */
    function isError($result)
    {
        if (method_exists($result, 'isError') && $result->isError()) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * quote
     *
     * @access private
     * @param mixed $value
     * @return string
     * @ignore
     */
    function quote($value)
    {
        return "\"$value\"";
    }

    /**
     * quoteId
     *
     * @access private
     * @param mixed $value
     * @return string
     * @ignore
     */
    function quoteId($value)
    {
        return "\"$value\"";
    }

}

?>
