<?php
/**
 * YANA library
 *
 * @package yana
 * @license http://creativecommons.org/licenses/by-nc-sa/2.5/
 */

/**
 * utility controller
 *
 * this implements the main program
 *
 * @static
 * @access      public
 * @name        Index
 * @package     yana
 * @subpackage  core 
 */
class Index extends Utility
{
    /**
     * YANA controller
     *
     * The Static function "main" is an YANA controller,
     * used to untaint user data and operate an YANA-instance.
     *
     * @static
     * @access  public
     * @uses    YANA::main()
     */
    function main()
    {
        /* Set error reporting to 'off' by default */
        if (!defined('YANA_ERROR_REPORTING')) {
            ErrorUtility::setErrorReporting(YANA_ERROR_OFF);
        }
        global $YANA;
        /* session preparation
         *
         * {@internal
         *
         * NOTE on session handling in PHP:
         *
         * As a default PHP marks a session as "expired" after 24 minutes,
         * BUT this does not mean the session data really gets deleted
         * immediatelly after this time!
         *
         * Session data only gets deleted if and only if it is marked
         * as expired AND the garbage collector is started.
         * Which MIGHT be (in best case) as soon as the data expires,
         * or (in worst case) never.
         *
         * The garbage collector is randomly started on an incoming request.
         * The propability in percent that the gc runs on a request
         * is calculated by (session.gc_probability / session.gc_divisor * 100)
         * Where gc_probability defaults to 1 and gc_divisor defaults
         * to 100, which actually results in 1% and means the gc
         * should run once each 100 requests.
         *
         * Of course this value does'nt fit all.
         * While it should work well on servers with frequent requests,
         * on a server with low traffic you may come back after days
         * or even weeks and find your session is still there.
         *
         * If you got low traffic you should consider increasing the value
         * of "session.gc_probability" from default 1 to a higher value
         * e.g. 5. Which actually means the garbage collector will autostart
         * each 5% of all incomming requests.
         *
         * Example: ini_set("session.gc_probability", "5");
         *
         * Don't use too high values either as this might slow
         * down your server. E.g. running the gc on every request is
         * certainly a bad idea.
         *
         * Abbr: "gc" = garbage collector
         * }}
         */
        /* set session lifetime to 30 minutes */
        ini_set("session.cookie_lifetime", "1800");
        ini_set("session.gc_maxlifetime", "1800");
        session_name('ysid');
        session_start();
        /* Check language settings */
        if (isset($_GET['language'])) {
            $_SESSION['language'] = $_GET['language'];
            unset($_GET['language']);
        }
        /* {@internal
         *
         * NOTE:
         * PHP's policy on which vars to import in what order to the super-global
         * $_REQUEST-var could have been changed in php.ini. Thus it might contain
         * other values than expected. This work-around ensures the expected behaviour.
         *
         * }}
         */
        $ARGS = mergeArrays($_GET, $_POST);
        /* {@internal
         *
         * Things to know BEFORE enabling output compression:
         * 1) headers already sent (e.g. by an PHP message, or another script?)
         * 2) did the browser provide a list of accepted encoding?
         * 3) does the browser accept the desired encoding?
         * 4) you can't use 'ob_gzhandler' if 'zlib.output_compression' is already enabled in php.ini
         *
         * Things to know AFTER enabling output compression:
         * 1) flushes automatically on program termination
         * 2) buffer can be erased using ob_clean() and ob_end_clean()
         * 3) flush buffer to output device using ob_flush() and ob_end_flush()
         * 4) use ob_get_length() === false to check if buffer is disabled
         * 5) use ob_get_length() === 0 to check if buffer is empty
         *
         * }}
         */
        if (headers_sent() === false && !ini_get('zlib.output_compression') && isset($_SERVER['HTTP_ACCEPT_ENCODING']) && strpos($_SERVER['HTTP_ACCEPT_ENCODING'],"gzip") !== false) {
            /* output compression level is an integer between -1 (off/default) - 9 (max) */
            ini_set('zlib.output_compression_level', 6);
            ob_start("ob_gzhandler");
        }
        /* Get a new yana-instance */
        $YANA = new Yana("config/system.config", $ARGS);
        if (isset($_SESSION['language']) && preg_match('/^[\w\d-_]+$/s',$_SESSION['language'])) {
            if (!$YANA->language->setDirectory($_SESSION['language'])) {
                unset($_SESSION['language']);
            }
        }
        /* Handle the request */
        $YANA->handle();
        /* Create the output */
        $YANA->writeView();
        /* {@internal
         *
         * Though this should'nt be necessary, one would be
         * best adviced to flush the buffer
         *
         * }}
         */
        if (ob_get_length() !== false) {
            ob_end_flush();
        }
    }

}

?>