<?php
/**
 * YANA library
 *
 * @package yana
 * @license http://creativecommons.org/licenses/by-nc-sa/2.5/
 */

/**
 * abstract Plugin
 *
 * This is an abstract super-class for all plugins.
 * Each subclass should re-implement the abstract
 * default event handler.
 *
 * @abstract
 * @access      public
 * @package     yana
 * @subpackage  core
 */
class Plugin
{

    /**
     * @ignore
     * @access  private
     * @var     string */ var $name = "";

    /**
     * constructor
     *
     * Creates a new instance.
     * When reimplementing this function,
     * note that you should at least set
     * the private name attribute, as this
     * serves as an id and therefore it
     * is essential to communicate with other
     * plugins.
     *
     * @param string $name
     */
    function Plugin($name)
    {
        settype($name,"string");
        assert('!empty($name);');
        $this->name = $name;
    }

    /**
     * abstract  default event handler
     *
     * Since this is an abstract function it should
     * be reimplemented in all subclasses
     *
     * @access public
     * @abstract
     * @param string $event
     * @param array $ARGS
     * @return boolean
     */
    function _default ($event, $ARGS)
    {
        /* intentionaly left blank */
        return true;
    }

    /**
     * get the name of a plugin
     *
     * The "name" of a plugin is also a unique identifier
     * for it. This means it is an alphanumeric, lower-cased string.
     * Plugin names can also be used to communicate with the
     * PluginManager singleton class to reference to a specific
     * plugin.
     *
     * @access public
     * @return string
     */
    function getName()
    {
        assert('is_string($this->name) && !empty($this->name);');
        return $this->name;
    }

}

?>