<?php
/**
 * YANA library
 *
 * @package yana
 * @license http://creativecommons.org/licenses/by-nc-sa/2.5/
 */

/**
 * Message
 *
 * This is a super class for all common messages.
 * It may be used to represent information, that
 * is not an error. E.g. reporting, that a certain
 * operation has been finished successfully.
 *
 * @access      public
 * @package     yana
 * @subpackage  error_reporting
 */
class Message extends Report
{
    /**
     * constructor
     *
     * @name    Message::__construct()
     * @param   array|string  $message
     * @param   mixed         $data    (optional)
     */
    function message($message, $data = null)
    {
        /* Implicitely allow int as input, so constants may be used that are resolved later at runtime */
        if (is_string($message) || is_int($message)) {
            settype($message,'string');
            $this->message = $message;
        } else if (is_array($message)) {
            if (count($message) == 2 && isset($message[0]) && isset($message[1])) {
                $this->message = array('H'=>$message[0],'P'=>$message[1]);
            } else {
                $message = changeCaseArray($message,CASE_UPPER);
                if (count($message) == 2 && isset($message['H']) && isset($message['P'])) {
                    $this->message = $message;
                } else {
                    trigger_error("Wrong argument type for argument 1 in ".__CLASS__."->".__FUNCTION__."().\n\t\tExpected a string or an array with the 2 elements, the first being the message's name and the second being the message description.\n\t\tInstead found '".gettype($message)."(".print_r($message,1).")'");
                }
            }
        } else {
            trigger_error("Wrong argument type for argument 1 in ".__CLASS__."->".__FUNCTION__."(). String or array expected, found '".gettype($message)."' instead.");
        }
        $this->data = $data;
    }

    /**
     * getMessage
     *
     * returns the message string of this event
     *
     * <ol>
     *     <li>    checks whether message is fulltext or id and resolves id,            </li>
     *     <li>    checks whether data is array or empty or other and replaces token    </li>
     * </ol>
     *
     * @access  public
     * @name    Message::getMessage()
     * @return  mixed
     */
    function getMessage()
    {
        global $YANA;
        $message = "";

        /* 1) */
        if (is_string($this->message) && preg_match('/^[\w_\d]+$/', $this->message)) {
            $message = $YANA->getVar('TEXT.'.strtoupper($this->message));
        } else {
            $message = $this->message;
        }
        if (empty($message)) {
            $message = $this->message;
        /* 2) */
        } else if (is_array($message) && !empty($this->data) && is_array($this->data)) {
            if (!empty($message['H'])) {
                $message['H'] = Layout::fillTemplate($message['H'], $this->data);
            }
            if (!empty($message['P'])) {
                $message['P'] = Layout::fillTemplate($message['P'], $this->data);
            }
        }
        return $message;
    }
}

?>