<?php
/**
 * YANA library
 *
 * @package yana
 * @subpackage vdrive
 * @license http://creativecommons.org/licenses/by-nc-sa/2.5/
 */

/**
 * VDrive_file
 *
 * class representing virtual files
 *
 * @access     public
 * @package    yana
 * @subpackage vdrive
 *
 * @ignore
 */
class VDrive_file extends VDrive_mountpoint
{

    /**
     * constructor
     *
     * @param  array   $SRC
     * @param  string  $type
     * @param  bool    $automount
     */
    function VDrive_file ($SRC, $type, $automount)
    {
        settype($SRC, "array");
        settype($type, "string");
        settype($automount, "boolean");

        if ($type === "file" || empty($type)) {
            $type = "secureInputStream";
        }

        if (preg_match('/^sml ?([\w_]*)$/i', $type, $m)) {
            $type = "SML";
            switch ($m[1])
            {
                case 'CASE_UPPER':
                    $case_sensitive = CASE_UPPER;
                break;
                case 'CASE_LOWER':
                    $case_sensitive = CASE_LOWER;
                break;
                default:
                    $case_sensitive = CASE_MIXED;
                break;
            }
        } else {
            $case_sensitive = null;
        }

        for ($i=0; $i<count($SRC) && class_exists($type) && (!method_exists($this->mountpoint, "exists") || !$this->mountpoint->exists()); $i++)
        {
            if (isset($case_sensitive)) {
                $this->mountpoint = new $type($SRC[$i], $case_sensitive);
            } else {
                $this->mountpoint = new $type($SRC[$i]);
            }
        } /* end for */

        if (!class_exists($type) || ( !is_subclass_of($this->mountpoint,"InputStream") && strtolower($type) != "buffereddbstream" && !is_subclass_of($this->mountpoint,"bufferedDbStream"))) {
            trigger_error("Invalid file-type argument '$type' supplied for Mountpoint '".print_r($SRC[$i],true)."'.", E_USER_WARNING);
            unset($this->mountpoint);
        } else {
            $this->type = $type;
            if ($automount) {
                $this->mount();
            }
        }

    }

}

?>