<?php
/**
 * @author     Thomas Meyer
 * @license    http://creativecommons.org/licenses/by-nc-sa/2.5/
 * @package    yana
 * @subpackage plugins
 */

/**
 * guestbook plugin
 *
 * @access     public
 * @package    yana
 * @subpackage plugins
 */
class plugin_guestbook extends Plugin
{

    /**#@+
     * @access  private
     * @ignore
     */

    /** @var mixed             */ var $database             = null;
    /** @var string (readonly) */ var $ACTION_SHOW          = "guestbook_read_read";
    /** @var string (readonly) */ var $ACTION_RSS           = "guestbook_read_rss";
    /** @var string (readonly) */ var $ACTION_DETAILS       = "guestbook_read_read_seperated";
    /** @var string (readonly) */ var $ACTION_NEW           = "guestbook_default_new";
    /** @var string (readonly) */ var $ACTION_NEW_WRITE     = "guestbook_write_new";
    /** @var string (readonly) */ var $ACTION_DELETE        = "guestbook_write_delete";
    /** @var string (readonly) */ var $ACTION_EDIT          = "guestbook_read_edit";
    /** @var string (readonly) */ var $ACTION_EDIT_WRITE    = "guestbook_write_edit";
    /** @var string (readonly) */ var $ACTION_COMMENT       = "guestbook_default_comment";
    /** @var string (readonly) */ var $ACTION_COMMENT_WRITE = "guestbook_write_comment";

    /**#@-*/

    /**
     * Constructor
     *
     * @param string $name a name for this plugin
     * @access public
     * @ignore
     */
    function plugin_guestbook($name)
    {
        settype($name,"string");
        global $YANA;

        $this->name = $name;
        $this->database = $YANA->connect('guestbook');

        /* Plugin-specific addities */
        $url = $YANA->getVar("SETTINGS.CSS");
        if (!empty($url)) {
            $YANA->setVar("CSS", DisplayUtility::css($url));
        }

        $YANA->setVar("ACTION_SHOW"         , $this->ACTION_SHOW);
        $YANA->setVar("ACTION_RSS"          , $this->ACTION_RSS);
        $YANA->setVar("ACTION_DETAILS"      , $this->ACTION_DETAILS);
        $YANA->setVar("ACTION_NEW"          , $this->ACTION_NEW);
        $YANA->setVar("ACTION_NEW_WRITE"    , $this->ACTION_NEW_WRITE);
        $YANA->setVar("ACTION_DELETE"       , $this->ACTION_DELETE);
        $YANA->setVar("ACTION_EDIT"         , $this->ACTION_EDIT);
        $YANA->setVar("ACTION_EDIT_WRITE"   , $this->ACTION_EDIT_WRITE);
        $YANA->setVar("ACTION_COMMENT"      , $this->ACTION_COMMENT);
        $YANA->setVar("ACTION_COMMENT_WRITE", $this->ACTION_COMMENT_WRITE);
    }

    /**
     * Default event handler
     *
     * returns bool(true) on success and bool(false) on error
     *
     * @access  public
     * @return  bool
     * @param   string  $event  name of the called event in lower-case
     * @param   array   $ARGS   array of arguments passed to the function
     * @ignore
     */
    function _default($event, $ARGS)
    {
        if (class_exists('Microsummary')) {
            Microsummary::publish($this->name);
        }
        /* check input data */
        settype($event, "string");
        settype($ARGS, "array");
        $event = strtolower($event);

        global $YANA;

        switch ($event)
        {

            /* Type: read, Permission: 75, Templates:  GUESTBOOK_EDIT */
            case $this->ACTION_EDIT:
                /* intentionally left blank */
                return true;
            break;

            /* Type: write, Permission: 75, Templates: MESSAGE  */
            case $this->ACTION_EDIT_WRITE:
                /* convert raw form data into arrays */
                $ENTRIES = array();
                foreach($ARGS['selected_entries'] as $id)
                {
                    $id = strtolower($id);
                    /* before doing anything, check if entry exists */
                    if (!$this->database->exists("guestbook.$id")) {
                        $YANA->report(new alert("ungueltige_eingabe"));
                        $YANA->report(new log("The selected entry guestbook.$id does not exist!"));
                        return false;
                    } else {
                        $new_entry = array();
                        foreach($ARGS as $key => $value)
                        {
                            if (preg_match('/^id_'.$id.'_[\w\d-_]+$/', $key)) {
                                $new_entry[str_replace("id_${id}_","", $key)] = $value;
                            }
                        } /* end foreach */
                        if (!empty($new_entry)) {
                            if (!$this->database->insert("guestbook.$id", $new_entry)) {
                                $YANA->report(new alert("ungueltige_eingabe"));
                                return false;
                            }
                        }
                    } /* end foreach */
                }
                /* don't forget to save your recent changes ;-) */
                if ($this->database->write()) {
                    return true;
                } else {
                    return false;
                }
            break;

            /* Type: write, Permission: 75, Templates: MESSAGE  */
            case $this->ACTION_DELETE:
                /* find any previously marked entries and mark them for removal */
                $ENTRIES = array();
                foreach($ARGS['selected_entries'] as $id)
                {
                    $id = strtolower($id);
                    /* before doing anything, check if entry exists */
                    if (!$this->database->exists("guestbook.$id")) {
                        $YANA->report(new alert("ungueltige_eingabe"));
                        $YANA->report(new log("The selected entry guestbook.$id does not exist!"));
                        return false;
                    } else {
                        if (!$this->database->remove("guestbook.$id")) {
                            $YANA->report(new alert("ungueltige_eingabe"));
                            return false;
                        }
                    }
                } /* end foreach */
                /* now delete those entries */
                if ($this->database->write()) {
                    return true;
                } else {
                    return false;
                }
            break;

            /* Type: default, Permission: 0, Templates:  GUESTBOOK_NEW */
            case $this->ACTION_NEW:
               $YANA->setVar('RSS_ACTION',$this->ACTION_RSS);
                if ($this->_securityCheck() === false) {
                    return false;
                }
                $YANA->setVar("SECURITY_IMAGE_INDEX", rand(1,9));
                $YANA->setVar("BESCHREIBUNG", '%LANGUAGE.DESCR_NEW%');
                $YANA->setVar('RSS_ACTION',$this->ACTION_RSS);
                return true;
            break;

            /* Type: read, Permission: 0, Templates:  NULL */
            case $this->ACTION_RSS:
                if (!class_exists('RSS')) {
                    trigger_error("RSS wrapper class not found. Unable to proceed.", E_USER_ERROR);
                    return false;
                } else {
                    if ($this->_securityCheck() === false) {
                        return false;
                    }
                    
                    /* get entries */
                    $args = array();
                    $args['sort']  = "guestbook_date";
                    $args['desc']  = true;
                    $profile_id    = $YANA->getVar("PROFILE.GUESTBOOK.PROFILE");
                    if (empty($profile_id)) {
                        $profile_id = 'default';
                    }
                    $args['where'] = array(array("profile_id",$profile_id,"="));
                    $args['page']  = 0;
                    $args['entries'] = 10;
                    $rows          = $this->_get_table($args, "guestbook");

                    /* create RSS feed */
                    $rss = new RSS();
                    $YANA->language->read('guestbook');
                    $rss->description = $YANA->getVar('LANGUAGE.RSS_DESCRIPTION');
                    if (empty($rss->description)) {
                        $rss->description = 'the 10 most recent guestbook entries';
                    }
                    $i = 0;
                    DisplayUtility::loadSmilies();
                    foreach ($rows as $row)
                    {
                        $item = new RSSitem();
                        /* 1) process title */
                        $item->title       = $row['GUESTBOOK_NAME'];
                        /* 2) process link */
                        $item->link        = DisplayUtility::url('action='.$this->ACTION_SHOW.'&entries=1&page='.$i, true).'#'.$i;
                        /**
                         * {@internal
                         * Note: guid is ignored by some aggregators and link is used instead
                         * where this is the case, the session id can not be part of the uri as this
                         * is subject to change. }}
                         */
                        $item->link        = str_replace(session_name()."=".session_id(),'',$item->link);
                        /* 3) process description */
                        $item->description = $row['GUESTBOOK_MESSAGE'];
                        $item->description = DisplayUtility::embeddedTags($item->description);
                        $item->description = DisplayUtility::smilies($item->description);
                        $item->description = strip_tags($item->description);
                        if (strlen($item->description) > 500) {
                            $item->description = substr($item->description,0,496).' ...';
                        }
                        /* 4) process pubDate */
                        $item->pubDate     = $row['GUESTBOOK_DATE'];
                        if (is_numeric($item->pubDate)) {
                            $item->pubDate = date('r', $item->pubDate);
                        }
                        $rss->addItem($item);
                        $i++;
                    } /* end foreach */
                    print utf8_encode($rss->toString());
                    exit(0);
                    return true;
                }
            break;

            /* Type: write, Permission: 0, Templates: MESSAGE  */
            case $this->ACTION_NEW_WRITE:
                if ($this->_securityCheck() === false) {
                    return false;
                }

                $permission = $YANA->getVar("PERMISSION");
                /* avoid spamming */
                if (!is_int($permission) || $permission < 1) {
                    if ($YANA->getVar("PROFILE.GUESTBOOK.SPAMPROTECT") === true && !$YANA->handle("security_check_image", $ARGS)) {
                        $YANA->report(new alert('ungueltige_eingabe', 'Security check failed!'));
                        $YANA->report(new log('SPAM: CAPTCHA not solved, entry has not been created.'));
                        return false;
                    }
                }

                /* get values from form fields to create new entry */
                $NEW_ENTRY = array();
                $profile_id = $YANA->getVar("PROFILE.GUESTBOOK.PROFILE");
                if (!empty($profile_id)){
                    $NEW_ENTRY['profile_id']          = $profile_id;
                } else if (!empty($ARGS['id'])) {
                    $NEW_ENTRY['profile_id']          = $ARGS['id'];
                } else {
                    $NEW_ENTRY['profile_id']          = 'default';
                }
                if (!empty($ARGS['guestbook_ip'])) {
                    $NEW_ENTRY['guestbook_ip']        = $ARGS['guestbook_ip'];
                }
                if (!empty($ARGS['guestbook_name'])) {
                    $NEW_ENTRY['guestbook_name']      = $ARGS['guestbook_name'];
                }
                if (!empty($ARGS['guestbook_message'])) {
                    $NEW_ENTRY['guestbook_message']   = $ARGS['guestbook_message'];
                }
                if (!empty($ARGS['guestbook_mail'])) {
                    $NEW_ENTRY['guestbook_mail']      = $ARGS['guestbook_mail'];
                }
                if (!empty($ARGS['guestbook_hometown'])) {
                    $NEW_ENTRY['guestbook_hometown']  = $ARGS['guestbook_hometown'];
                }
                if (!empty($ARGS['guestbook_homepage'])) {
                    $NEW_ENTRY['guestbook_homepage']  = $ARGS['guestbook_homepage'];
                }
                if (!empty($ARGS['guestbook_messenger'])) {
                    $NEW_ENTRY['guestbook_messenger'] = $ARGS['guestbook_messenger'];
                }
                if (!empty($ARGS['guestbook_msgtyp'])) {
                    $NEW_ENTRY['guestbook_msgtyp']    = $ARGS['guestbook_msgtyp'];
                }
                if (!empty($ARGS['guestbook_opinion'])) {
                    $NEW_ENTRY['guestbook_opinion']   = $ARGS['guestbook_opinion'];
                }

                plugin_guestbook::_untaintInput($NEW_ENTRY);
                if (empty($NEW_ENTRY)) {
                    return false;
                }

                /* mark registered users */
                if (strcasecmp($YANA->getVar("SESSION_USER_ID"), $ARGS['guestbook_name']) == 0) {
                    $NEW_ENTRY['guestbook_is_registered'] = 1;
                }

                /* avoid flooding */
                $myFlood = $YANA->plugins->get($this->name, "flood");
                $myFlood->setMax((int)$YANA->getVar("PROFILE.GUESTBOOK.FLOODING"));
                if ($myFlood->isBlocked()) {
                    $YANA->report(new log('Possibly flooding attempt detected. User request rejected.'));
                    $YANA->report(new alert('flooding'));
                    return false;
                }
                $recent_entry = $this->database->get("guestbook.?.guestbook_message");
                if(!empty($NEW_ENTRY['guestbook_message']) && $recent_entry == $NEW_ENTRY['guestbook_message']) {
                    $YANA->report(new alert('nospam'));
                    return false;
                } elseif($myFlood->getMax()>0) {
                    $myFlood->set();
                }

                /* insert new entry into table */
                if (!$this->database->insert('guestbook.*',$NEW_ENTRY)) {
                    $YANA->report(new log('Failed to insert entry.', preg_replace('/\s/', ' ', SML::encode($NEW_ENTRY)) ));
                    $YANA->report(new alert('ungueltige_eingabe'));
                    return false;
                } elseif (!$this->database->write()) {
                    $YANA->report(new log('Unable to submit entry.', preg_replace('/\s/', ' ', SML::encode($NEW_ENTRY)) ));
                    return false;
                } else {
                    /* send Mail */
                    if (!empty($NEW_ENTRY['guestbook_mail']) && $YANA->getVar("PROFILE.GUESTBOOK.SENDMAIL")) {
                        $this->_send_mail($NEW_ENTRY['guestbook_mail'],$YANA->skin->getFile("danke_mail"),$ARGS);
                    }
                    if ($YANA->getVar("PROFILE.GUESTBOOK.MAIL") && $YANA->getVar("PROFILE.GUESTBOOK.NOTIFICATION")) {
                        $this->_send_mail($YANA->getVar("PROFILE.GUESTBOOK.MAIL"),$YANA->skin->getFile("benachrichtigung_mail"),$ARGS);
                    }
                    if (class_exists('Microsummary')) {
                        Microsummary::set($this->name, strtoupper($this->name).', update '.date('d M y G:s',time()));
                    }
                    return true;
                }
            break;

            /* Type: read, Permission: 0, Templates:  GUESTBOOK_READ */
            case $this->ACTION_SHOW:
                if ($this->_securityCheck() === false) {
                    return false;
                }
                $ARGS['sort']  = "guestbook_date";
                $ARGS['desc']  = true;
                $profile_id    = $YANA->getVar("PROFILE.GUESTBOOK.PROFILE");
                if (empty($profile_id)) {
                    $profile_id = ( (!empty($ARGS['id'])) ? $ARGS['id'] : 'default');
                }
                $ARGS['where'] = array(array("profile_id",$profile_id,"="));
                $YANA->setVar('DATABASE', changeCaseArray($this->database->structure->getStructure(),CASE_UPPER));
                $YANA->setVar('ROWS', $this->_get_table($ARGS, "guestbook"));
                $YANA->setVar("BESCHREIBUNG", '%LANGUAGE.DESCR_SHOW%');
                $YANA->setVar('RSS_ACTION',$this->ACTION_RSS);
                return true;
            break;


            /* Type: read, Permission: 0, Templates:  GUESTBOOK_READ_SEPERATED */
            case $this->ACTION_DETAILS:
                if ($this->_securityCheck() === false) {
                    return false;
                }
                $ARGS['sort']  = "guestbook_date";
                $ARGS['desc']  = true;
                $profile_id    = $YANA->getVar("PROFILE.GUESTBOOK.PROFILE");
                if (empty($profile_id)) {
                    $profile_id = ( (!empty($ARGS['id'])) ? $ARGS['id'] : 'default');
                }
                $ARGS['where'] = array(array("profile_id", $profile_id, "="));
                $YANA->setVar('DATABASE', changeCaseArray($this->database->structure->getStructure(),CASE_UPPER));
                $YANA->setVar('ROWS', $this->_get_table($ARGS, "guestbook"));
                $YANA->setVar("BESCHREIBUNG", '%LANGUAGE.DESCR_DETAILS%');
                $YANA->setVar('RSS_ACTION',$this->ACTION_RSS);
                return true;
            break;

            /* Type: default, Permission: 30, Templates: GUESTBOOK_DEFAULT_COMMENT  */
            case $this->ACTION_COMMENT:
               if (!is_numeric($ARGS['target'])) {
                   $YANA->message('alert', 'ungueltige_eingabe', $this->ACTION_SHOW);
                   return false;
               } else {
                   settype($ARGS['target'], 'int');
                   $guestbook_comment = $this->database->get('guestbook.'.$ARGS['target'].'.guestbook_comment');
                   if (!empty($guestbook_comment)) {
                        $YANA->setVar('GUESTBOOK_COMMENT', $guestbook_comment);
                   }
                   return true;
               }
            break;


            /* Type: write, Permission: 30, Templates: MESSAGE  */
            case $this->ACTION_COMMENT_WRITE:
               if (!is_numeric($ARGS['target']) || !is_string($ARGS['guestbook_comment'])) {
                   $YANA->message('alert', 'ungueltige_eingabe', $this->ACTION_COMMENT);
                   return false;
               } else if (!$this->database->insert('guestbook.'.$ARGS['target'].'.guestbook_comment', $ARGS['guestbook_comment']) ||
                   !$this->database->write()) {
                   $YANA->report(new log('Unable to insert comment.', $ARGS['guestbook_comment']));
                   return false;
               } else {
                   return true;
               }
            break;

            default:
                /* intentionally left blank */
            break;

        } /* end switch */

        return true;
    }

    function _get_table($ARGS, $table)
    {
        global $YANA;

        @settype($table,           "string");
        if (!isset($ARGS['where']) || (!is_string($ARGS['where']) && !is_array($ARGS['where']))) {
            $ARGS['where'] = "";
        }
        @settype($ARGS['sort'],    "string");
        @settype($ARGS['page'],    "integer");
        @settype($ARGS['entries'], "integer");

        /* set up vars */
        $table       = strtolower($table);
        $primary_key = "";
        if (isset($ARGS['desc'])) {
            @settype($ARGS['desc'],   "boolean");
            $desc      = ($ARGS['desc']) ? true : false;
        } else {
            $desc      = true;
        }
        if ($ARGS['page']<0) {
            $page        = 0;
        } else { 
            $page        = (int) $ARGS['page'];
        }
        $entPerPage      = (int) $ARGS['entries'];
        if ($entPerPage<1) {
            $entPerPage  = (int) $YANA->getVar("PROFILE.GUESTBOOK.ENTPERPAGE");
        }
        if ($entPerPage<1) {
            $entPerPage  = 10;
        }
        $primary_key = $this->database->structure->getPrimaryKey($table);
        if (!empty($ARGS['sort'])) {
            $sortBy      = $ARGS['sort'];
        } else {
            $sortBy      = $primary_key;
        }

        $result          = $this->database->get($table, $ARGS['where'], $sortBy, $page, $entPerPage, $desc);

        $YANA->setVar('TABLE',       $table);
        $YANA->setVar('PRIMARY_KEY', $primary_key);
        $last_page = $this->database->length($table, $ARGS['where']);
        $YANA->setVar('LAST_PAGE', $last_page);
        $YANA->setVar('SORT', $sortBy);
        /* deprecated */
        $YANA->setVar('WHERE', '');
        $liste = array();
        if ($last_page > ($entPerPage*2)) {
            $dots = false;
            for($k=0;$k<ceil($last_page/$entPerPage);$k++) 
            {
                $it   = count($liste);
                $first = ($k*$entPerPage);
                $last  = ((($k+1)*$entPerPage < $last_page) ? ($k+1)*$entPerPage : $last_page);
                if ($last_page > 10 * $entPerPage && $k > 1 && $k < ceil($last_page/$entPerPage) - 2 && !($first > $page-$entPerPage-1 && $last < $page+1+(2*$entPerPage))) {
                    if (!$dots) {
                        $dots = true;
                        $liste[$it]['TOO_MANY'] = true;
                    }
                } else {
                    $liste[$it]['FIRST'] = ($first+1);
                    $liste[$it]['LAST']  = $last;
                    if ($first > $page-1 && $last < $page+$entPerPage+1) {
                        $dots = false;
                    }    
                }
            } /* end foreach */
        }
        $YANA->setVar('LIST_OF_ENTRIES', $liste);
        $YANA->setVar('ENTRIES_PER_PAGE', $entPerPage);
        $YANA->setVar('DESC', $desc);
        $YANA->setVar('FIRST_PAGE', $page+1);
        $YANA->setVar('OFFSET_PAGE', (($page+$entPerPage>$last_page) ? $last_page : $page+$entPerPage));
        return $result;
    }

    /**
     * _securityCheck
     *
     * @access private
     * @return boolean
     * @ignore
     */
     function _securityCheck()
     {
        global $YANA;
        $id = strtolower($YANA->getVar('ID'));
        /* do not show new guestbooks, if Auto-Option is deactivated */
        $dir  = $YANA->registry->get('PROFILEDIR');
        $file = new inputStream($dir->getFilename().$id.'.cfg');
        if ($id !== 'default' && !$file->exists() && !$YANA->getVar("PROFILE.AUTO")) {
            $YANA->report(new log('Access restriction in effect. Access to undefined profile '.$id.' denied.'));
            $YANA->report(new error('4041'));
            return false;
        } else {
            return true;
        }
    }

    /**
     * _send_mail
     *
     * @access private
     * @param string $empfaenger
     * @param string $template
     * @param array $INPUT
     * @ignore
     */
    function _send_mail($empfaenger,$template,$INPUT)
    {
        assert('is_array($INPUT);');
        global $YANA;
        $INPUT = changeCaseArray($INPUT,CASE_UPPER);
        $heute         = getdate();
        $mail          = new mailer($template);
        $mail->subject = $YANA->getVar("LANGUAGE.MAIL_SUBJECT");
        $mail->insert("*", $INPUT);
        $mail->insert("*", $YANA->getVar("*"));
        $mail->insert('DATUM', $heute['mday'] . '.' . $heute['mon'] . '.' . $heute['year']);
        $mail->send($empfaenger);
     }

    /**
     * _untaintInput
     *
     * @access private
     * @param mixed $empfaenger
     * @ignore
     */
     function _untaintInput(&$input)
     {
        if (is_array($input)) {
            foreach ($input as $a => $b)
            {
                $input[$a] = untaintInput($b, "string", 0, YANA_ESCAPE_USERTEXT);
            }
        }
     }

}

//  2006, Thomas Meyer www.All-Community.de
// Copyright Hinweis nicht entfernen!
?>