<?php
/**
 * @author     Thomas Meyer
 * @license    http://creativecommons.org/licenses/by-nc-sa/2.5/
 * @package    yana
 * @subpackage plugins
 */

/**
 * IP block setup
 *
 * This implements basic setup functions for
 * the IP block plugin
 *
 * @access     public
 * @package    yana
 * @subpackage plugins
 */
class plugin_ipblock_admin extends Plugin
{

    /**#@+
     * @ignore
     * @access  private
     */

    /** @var blockfile */ var $whitelist;
    /** @var blockfile */ var $blacklist;
    /** @var dirStream */ var $dir;

    /**#@-*/

    /**
     * Constructor
     *
     * @param string $name a name for this plugin
     * @access public
     * @ignore
     */
    function plugin_ipblock_admin($name)
    {
        settype($name,"string");        
        global $YANA;
        
        $this->name      = $name;
        $this->whitelist = $YANA->plugins->get($this->name,"IPWHITELIST");
        $this->blacklist = $YANA->plugins->get($this->name,"IPBLACKLIST");
        $this->dir       = $YANA->plugins->get($this->name,"BLOCKDIR");
    }

    /**
     * Default event handler
     *
     * returns bool(true) on success and bool(false) on error
     *
     * @access  public
     * @return  bool
     * @param   string  $event  name of the called event in lower-case
     * @param   array   $ARGS   array of arguments passed to the function
     * @ignore
     */
    function _default($event, $ARGS)
    {
        settype($event, "string");
        settype($ARGS, "array");

        if ($this->blacklist && $this->whitelist) {
            if ($this->whitelist->exists()) {
                $this->whitelist->read();
            }
            if ($this->blacklist->exists()) {
                $this->blacklist->read();
            }

            global $YANA;

            switch ($event) {

                case "get_block": case "get_root_block":
                    $YANA->setVar("BLACKLIST", $this->blacklist->get());
                    $YANA->setVar("WHITELIST", $this->whitelist->get());
                    return true;
                break;

                case "set_block":
                    $this->blacklist = new blockfile($this->dir->getFilename().$ARGS['id'].".blacklist");
                    $this->whitelist = new blockfile($this->dir->getFilename().$ARGS['id'].".whitelist");

                    if ($this->whitelist->exists()) {
                        $this->whitelist->read();
                    }
                    if ($this->blacklist->exists()) {
                        $this->blacklist->read();
                    }

                    return $this->_set($ARGS);
                break;

                case "set_root_block":
                    return $this->_set($ARGS);
                break;

                default:
                    /* intentionally left blank */
                break;
            } /* end switch */
        }
        return true;
    }
    
    /**
     * _untaintInput
     *
     * @access private
     * @param  array $ARGS
     * @return boolean
     * @ignore
     */
    function _set($ARGS)
    {
         settype($ARGS, "array");
         global $YANA;

         if (!$this->whitelist->exists() && !$this->whitelist->create()) {
             return false;
         } else  if (!$this->blacklist->exists() && !$this->blacklist->create()) {
             return false;
         } else if ((!$this->blacklist->set($ARGS['blacklist']) || !$this->blacklist->write()) ||
             (!$this->whitelist->set($ARGS['whitelist']) || !$this->whitelist->write()))
         {
             return false;
         } else {
             return true;
         }
    }

}

//  2006, Thomas Meyer www.All-Community.de
// Copyright Hinweis nicht entfernen!
?>