<?php
/**
 * @author     Thomas Meyer
 * @license    http://creativecommons.org/licenses/by-nc-sa/2.5/
 * @package    yana
 * @subpackage plugins
 */

/**
 * Software Developement Kit
 *
 * This implements a code generator utility
 *
 * @access     public
 * @package    yana
 * @subpackage plugins
 */
class plugin_sdk extends Plugin
{

    /**
     * Default event handler
     *
     * returns bool(true) on success and bool(false) on error
     *
     * @access  public
     * @return  bool
     * @param   string  $event  name of the called event in lower-case
     * @param   array   $ARGS   array of arguments passed to the function
     * @ignore
     */
    function _default($event, $ARGS)
    {
        global $YANA;
        /* do nothing */
        return true;
    }

    /**
     * Create form
     *
     * Type: read
     * Permission: 100
     * Templates:  sdk_read_form
     *
     * @param array $ARGS array of params passed to the function
     * @return boolean
     * @ignore
     */
    function sdk_read_form ($ARGS) 
    {
        global $YANA;
        $dir = $YANA->plugins->get($this->name, 'LOGOS');
        $dir->read();
        $YANA->setVar('FILES',   $dir->get());
        $YANA->setVar('IMG_SRC', $dir->getFilename());
        $config = $YANA->plugins->get($this->name, 'CONFIGFILE');
        $config->read();
        $YANA->setVar('PLUGIN',  $config->get());
    }

    /**
     * create plugin
     *
     * Type: write
     * Permission: 100
     * Templates: MESSAGE
     *
     * @param array $ARGS array of params passed to the function
     * @return boolean
     * @ignore
     */
    function sdk_write_plugin ($ARGS) 
    {
        global $YANA;

        $this->_createBackup($ARGS);

        /* Initialize vars */
        $INFO             = array();
        $DRIVE            = array();
        $CUSTOM_INTERFACE = array();
        $AUTO_INTERFACE   = array();
        $INTERFACE        = array();
        $COPY             = array();
        $DATABASE         = array();
        $SKIN             = array();

        /* Check user input */
        $is_valid = true;

        /* NAME */
        if (!is_string($ARGS['name']) || strlen($ARGS['name']) > 15 || !preg_match('/^[\d\w-_ \(\)]+$/si', $ARGS['name'])) {
            $YANA->report(new error("INVALID_CHARACTERS", array('FIELD'=>'NAME','VALUE'=>print_r($ARGS['name'],true),'VALID'=>'a-z, 0-9, -, _, , , , , " "')));
            /* Can't continue without valid id */
            return false;
        } else {
            $INFO['NAME'] = $ARGS['name'];
            $INFO['ID']   = preg_replace('/[^\d\w-_]/','_',strtolower($INFO['NAME']));
            unset($ARGS['name']);
        }

        /* load files */
        $dir     = $YANA->plugins->get($this->name, 'LOGOS');
        $plgDir  = new dirStream($YANA->getVar('PLUGINDIR').$INFO['ID'].'/');
        $sknDir  = new dirStream($YANA->getVar('SKINDIR').'default/'.$INFO['ID'].'/');
        $sknFile = new SML($YANA->getVar('SKINDIR').'default/'.$INFO['ID'].'.config', CASE_UPPER);
        $plgFile = new SML($YANA->getVar('PLUGINDIR').$INFO['ID'].'.config',          CASE_UPPER);
        $drvFile = new SML($plgDir->getFilename().'_drive.config',                    CASE_UPPER);
        $phpFile = new secureFileStream($plgDir->getFilename().'plugin.php');

        /* GRAFIK */
        if (!is_string($ARGS['grafik']) || !preg_match('/^[\d\w-_\.]+\.(gif|jpg|png)$/si', $ARGS['grafik'])) {
            $YANA->report(new error("INVALID_SYNTAX", array('FIELD'=>'LOGO','VALUE'=>print_r($ARGS['grafik'],true),'VALID'=>'file.gif, file.jpg, file.png')));
            $is_valid = false;
        } else {
            $endung = '.'.preg_replace('/^.*\./', '', $ARGS['grafik']);
            $i = count($COPY);
            $COPY[$i]['src']  = $dir->getFilename().$ARGS['grafik'];
            $COPY[$i]['dest'] = $plgDir->getFilename().'preview'.$endung;
            $INFO['GRAFIK'] = '%PLUGINDIR%/'.$INFO['ID'].'/'.'preview'.$endung;
            unset($endung, $i);
        }

        /* PARENT */
        if (!is_string($ARGS['parent']) || !preg_match('/^[\d\w-_]*$/si', $ARGS['parent'])) {
            $YANA->report(new error("INVALID_CHARACTERS", array('FIELD'=>'PARENT','VALUE'=>print_r($ARGS['parent'],true),'VALID'=>'a-z, 0-9, -, _')));
            $is_valid = false;
        } else {
            if (!empty($ARGS['parent'])) {
                $INFO['PARENT'] = strip_tags(nl2br($ARGS['parent']));
            }
            unset($ARGS['parent']);
        }

        /* PACKAGE */
        if (!is_string($ARGS['package']) || !preg_match('/^[\d\w-_]*$/si', $ARGS['package'])) {
            $YANA->report(new error("INVALID_CHARACTERS", array('FIELD'=>'PACKAGE','VALUE'=>print_r($ARGS['package'],true),'VALID'=>'a-z, 0-9, -, _')));
            $is_valid = false;
        } else {
            if (!empty($ARGS['package'])) {
                $INFO['PACKAGE'] = strip_tags(nl2br($ARGS['package']));
            }
            unset($ARGS['package']);
        }

        /* TYPE */
        if (!is_string($ARGS['type']) || !preg_match('/^(primary|default|config|read|write|security)$/si', $ARGS['type'])) {
            $YANA->report(new error("INVALID_SYNTAX", array('FIELD'=>'TYPE','VALUE'=>print_r($ARGS['type'],true),'VALID'=>'primary, default, config, read, write, security')));
            $is_valid = false;
        } else {
            $INFO['TYPE'] = strip_tags(nl2br($ARGS['type']));
            unset($ARGS['type']);
        }

        /* PRIORITY */
        if (!is_string($ARGS['priority']) || !is_numeric($ARGS['priority']) || (int) $ARGS['priority'] != (string) $ARGS['priority'] || $ARGS['priority'] < -1 || $ARGS['priority'] > 3) {
            $YANA->report(new error("INVALID_SYNTAX", array('FIELD'=>'PRIORITY','VALUE'=>print_r($ARGS['priority'],true),'VALID'=>'-1 - 3')));
            $is_valid = false;
        } else {
            $INFO['PRIORITY'] = (int) $ARGS['priority'];
            unset($ARGS['priority']);
        }

        /* AUTOR */
        if (!is_string($ARGS['autor'])) {
            $is_valid = false;
        } else {
            if (!empty($ARGS['autor'])) {
                $INFO['AUTOR'] = strip_tags(nl2br($ARGS['autor']));
            }
            unset($ARGS['autor']);
        }

        /* KONTAKT */
        if (!is_string($ARGS['kontakt'])) {
            $is_valid = false;
        } else {
            if (!empty($ARGS['kontakt'])) {
                $INFO['KONTAKT'] = strip_tags(nl2br($ARGS['kontakt']));
            }
            unset($ARGS['kontakt']);
        }


        /* BESCHREIBUNG */
        if (!is_string($ARGS['beschreibung'])) {
            $is_valid = false;
        } else {
            if (!empty($ARGS['beschreibung'])) {
                $INFO['BESCHREIBUNG'] = str_replace("\n",'[br]',strip_tags($ARGS['beschreibung']));
            }
            unset($ARGS['beschreibung']);
        }
    
        /* UPDATE */
        if (!is_string($ARGS['update'])) {
            $is_valid = false;
        } else {
            if (!empty($ARGS['update'])) {
                $INFO['UPDATE'] = strip_tags(nl2br($ARGS['update']));
            }
            unset($ARGS['update']);
        }

        /* if user input is invalid, exit with error */
        if ($is_valid === false) {
            $YANA->report(new error("UNGUELTIGE_EINGABE"));
            return false;
        }

        if (!empty($ARGS['overwrite'])) {
            if ($plgDir->exists()) {
                $plgDir->delete(true);
            }
            if ($sknDir->exists()) {
                $sknDir->delete(true);
            }
            if ($sknFile->exists()) {
                $sknFile->delete();
            }
            if ($plgFile->exists()) {
                $plgFile->delete();
            }    
        } elseif ($plgFile->exists() || $plgDir->exists()) {
            $YANA->report(new error("ALLREADY_EXISTS", array('ID'=>$INFO['ID'])));
            return false;
        }
        unset($ARGS['overwrite']);

        /* CREATE FILES */
        $plgFile->create();
        $plgDir->create(777);
        $drvFile->create();
        $phpFile->create();
        $sknDir->create(663);

        { /* sql_file */

            if (!empty($_FILES['sqlfile']['tmp_name'])) {
                move_uploaded_file($_FILES['sqlfile']['tmp_name'],$YANA->getVar('DBDIR').'.install/mysql/'.$INFO['ID'].'.sql');
                unset($ARGS['sqlfile']);
            } else {
            }

        }

        { /* CONFIG_TEMPLATE */
    
            if ($ARGS['config_template'] == 'true'/*  || $create_installation_routine */) {              

                /* create setup entry */
                $INFO['SETUP'] = array();
                $INFO['SETUP'][0] = array();
                $INFO['SETUP'][0]['ACTION'] = $INFO['ID'].'_config_setup';
                $INFO['SETUP'][0]['IMAGE']  = $INFO['ID'].'/setup.gif';
                $INFO['SETUP'][0]['TITLE']  = $INFO['NAME'];

                /* copy icon */
                $iconDir = $YANA->plugins->get($this->name, 'ICONS');
                $COPY[$i]['src']  = $iconDir->getFilename().preg_replace('/(.*)\..*/', "\\1", $ARGS['grafik']).'.gif';
                $COPY[$i]['dest'] = $plgDir->getFilename().'setup.gif';

            }
            unset($ARGS['config_template']);
            unset($ARGS['grafik']);
    
        }

        if (!empty($_FILES['sourcefile']['tmp_name'])) { /* source_file */
    
            move_uploaded_file($_FILES['sourcefile']['tmp_name'],$YANA->getVar('DBDIR').$INFO['ID'].'.config');
            unset($ARGS['sourcefile']);
    
            $dbsFile = new SML($YANA->getVar('DBDIR').$INFO['ID'].'.config', CASE_UPPER);
            $dbsFile->init();
            $DATABASE = $dbsFile->get('*');

        } else {      
    
            $DATABASE = array();
    
        }  
    
        { /* INTERFACE */
    
            if ($ARGS['interface']) {
                $raw_data = explode("\n", $ARGS['interface']);
    
                for ($i=0; $i<count($raw_data); $i++)
                {
                    $raw_data[$i] = explode(",",$raw_data[$i]);
                    $j = $raw_data[$i][0];
                    if (!empty($j)) {
                        if (isset($CUSTOM_INTERFACE[$j])) echo '<h1>Warnung</h1><p>Eine Aktion mit der Bezeichnung &quot;'.$j.'&quot; existiert mehrmals. Erste Version wurde verworfen.</p>';
                        $CUSTOM_INTERFACE[$j] = array();
                        $CUSTOM_INTERFACE[$j]['TYPE'] = $raw_data[$i][1];
                        $CUSTOM_INTERFACE[$j]['MODE'] = $raw_data[$i][2];
                        if(!empty($raw_data[$i][3])) {
                            $CUSTOM_INTERFACE[$j]['TEMPLATE'] = $raw_data[$i][3];
                        }
                        if(!empty($raw_data[$i][4])) {
                            $CUSTOM_INTERFACE[$j]['INSERT'] = $raw_data[$i][4];
                        }
                        $CUSTOM_INTERFACE[$j]['PERMISSION'] = (int) $raw_data[$i][5];
                    }
                } /* end for */
            }
            unset($ARGS['interface']);
            unset($ARGS['interface_mode']);
        }

        { /* create virtual drive */
        
            $DRIVE['MAIN'] = array();
            $DRIVE['MAIN']['TYPE']      = 'executeable';
            $DRIVE['MAIN']['REQUIRED']  = 'readable';
            $DRIVE['MAIN']['AUTOMOUNT'] = 'yes';
            $DRIVE['MAIN']['SRC']       = 'plugin.php';
    
        }

        { /* create GUI and corresponding Interface entries for each table */

            $INPUT = mergeArrays($INFO,array('INTERFACE' => $CUSTOM_INTERFACE));
            $HTML_MENU = array();
            $HTML_MENU['table_list'] = array();
            $HTML_MENU['event_list'] = array();

            if (isset($DATABASE['TABLES'])) {
                foreach ($DATABASE['TABLES'] as $KEY => $CURRENT)
                {
                    $KEY = strtolower($KEY);
                    array_push($HTML_MENU['table_list'], $KEY);
                    $DB_SRC = array('DATABASE' => $CURRENT['CONTENT'], 'TABLE' => $KEY, 'PRIMARY_KEY' => $CURRENT['PRIMARY_KEY']);
    
                    { /* Edit */
                        $current_action = $this->_create('gui_edit.html', $INFO['ID'], 'read', 'edit', $INPUT, $DB_SRC, 75, $sknDir, $AUTO_INTERFACE, $SKIN, $INTERFACE, $INFO);
                        $this->_create('', $INFO['ID'], 'write', 'edit', $INPUT, $DB_SRC, 75, $sknDir, $AUTO_INTERFACE, $SKIN, $INTERFACE, $INFO);
                        array_push($HTML_MENU['event_list'], array('action' => $current_action, 'permission' => 75, 'table_name' => $KEY, 'name' => 'edit'));
                    }
        
                    { /* Delete */
                        $this->_create('', $INFO['ID'], 'write', 'delete', $INPUT, $DB_SRC, 75, $sknDir, $AUTO_INTERFACE, $SKIN, $INTERFACE, $INFO);
                    }
        
                    { /* New */
                        $current_action = $this->_create('gui_new.html', $INFO['ID'], 'default', 'new', $INPUT, $DB_SRC, 0, $sknDir, $AUTO_INTERFACE, $SKIN, $INTERFACE, $INFO);
                        $this->_create('', $INFO['ID'], 'write', 'new', $INPUT, $DB_SRC, 0, $sknDir, $AUTO_INTERFACE, $SKIN, $INTERFACE, $INFO);
                        array_push($HTML_MENU['event_list'], array('action' => $current_action, 'permission' => 0, 'table_name' => $KEY, 'name' => 'new'));
                    }
        
                    { /* Search */
                        $current_action = $this->_create('gui_search.html', $INFO['ID'], 'read', 'search', $INPUT, $DB_SRC, 0, $sknDir, $AUTO_INTERFACE, $SKIN, $INTERFACE, $INFO);
                        array_push($HTML_MENU['event_list'], array('action' => $current_action, 'permission' => 0, 'table_name' => $KEY, 'name' => 'search'));
                    }
    
                    { /* View */
                        $current_action = $this->_create('gui_view.html', $INFO['ID'], 'read', 'read', $INPUT, $DB_SRC, 0, $sknDir, $AUTO_INTERFACE, $SKIN, $INTERFACE, $INFO);
                        array_push($HTML_MENU['event_list'], array('action' => $current_action, 'permission' => 0, 'table_name' => $KEY, 'name' => 'list'));
                    }
        
                    { /* View */
                        $current_action = $this->_create('gui_view_seperated.html', $INFO['ID'], 'read', 'read_seperated', $INPUT, $DB_SRC, 0, $sknDir, $AUTO_INTERFACE, $SKIN, $INTERFACE, $INFO);
                        array_push($HTML_MENU['event_list'], array('action' => $current_action, 'permission' => 0, 'table_name' => $KEY, 'name' => 'view'));
                    }
    
                } /* end foreach */
            } /* end if */
    
        }
        
        { /* create index file */

            $tplDir   = $YANA->plugins->get($this->name, 'TEMPLATES');
            $tplDir   = $tplDir->getFilename();
            $idxTemplate = new smartTemplate($tplDir.'gui_index.html');
            $idxTemplate->insert('*', $HTML_MENU);
            $idxTemplate->insert('plugin_id', $INFO['ID']);
            $IDX         =  $idxTemplate->get();

        }
    
        { /* create class skeleton */

            $classFile   = $YANA->plugins->get($this->name, 'CLASS');
            $phpTemplate = new smartTemplate($classFile->getFilename());
            $phpTemplate->insert('*', $INFO);
            $phpTemplate->insert('INTERFACE', $CUSTOM_INTERFACE);
            $phpTemplate->insert('AUTO_INTERFACE', $AUTO_INTERFACE);
            $phpTemplate->insert('DATABASE',$DATABASE);      
            $PHP         =  $phpTemplate->get();
    
        }
    
        { /* copy files */
    
            foreach ($COPY as $k => $v) 
            {
                if (!copy($v['src'], $v['dest'])) {
                    $YANA->report(new error('NOT_WRITEABLE', array('FILE'=>$v['dest'])));
                    return false;
                }
            }
            
        }
        
        { /* write files */
    
            $INTERFACE = mergeArrays($CUSTOM_INTERFACE, $INTERFACE);
            $plgFile->set(array('INFO' => $INFO, 'INTERFACE' => $INTERFACE));
            if (!$plgFile->write()) {
                $YANA->report(new error('NOT_WRITEABLE', array('FILE'=>$plgFile->getFilename())));
                return false;
            }

            $drvFile->set($DRIVE);
            if (!$drvFile->write()) {
                $YANA->report(new error('NOT_WRITEABLE', array('FILE'=>$drvFile->getFilename())));
                return false;
            }

            $phpFile->insert("<?php\n".$PHP."\n?>");
            if (!$phpFile->write()) {
                $YANA->report(new error('NOT_WRITEABLE', array('FILE'=>$phpFile->getFilename())));
                return false;
            }

        }

        { /* create Skin file */
    
            $sknFile->create();
            $sknFile->set($SKIN);
            if (!$sknFile->write()) {
                $YANA->report(new error('NOT_WRITEABLE', array('FILE'=>$sknFile->getFilename())));
                return false;
            }

    
        }

        { /* create Index file */

            $idxFile = new secureFileStream($sknDir->getFilename().'index.html');
            $idxFile->create();
            $idxFile->insert($IDX);
            if (!$idxFile->write()) {
                $YANA->report(new error('NOT_WRITEABLE', array('FILE'=>$idxFile->getFilename())));
                return false;
            }

        }

        return true;

    }


    /**
     * Reverse-engineer database
     *
     * Type: write
     * Permission: 100
     * Templates: MESSAGE
     *
     * @param array $ARGS array of params passed to the function
     * @return boolean
     * @ignore
     */
    function sdk_write_config ($ARGS)
    {
        global $YANA;

        $this->_createBackup($ARGS);

        $db = new bufferedDbStream();
        if ($db->exportStructure($ARGS["table"])) {
            return true;
        } else {
            $YANA->report(new error("FILE_NOT_CREATED", array('FILE'=>$ARGS["table"])));
            return false;
        }
    }

    /**
     * @access private
     * @ignore
     */
    function _create ($src, $id, $type, $name, &$INPUT, &$DATABASE, $mode = 0, &$sknDir, &$AUTO_INTERFACE, &$SKIN, &$INTERFACE, &$INFO) 
    {
        global $YANA;
        $tplDir   = $YANA->plugins->get($this->name, 'TEMPLATES');
        $tplDir   = $tplDir->getFilename();
        $action   = $this->_createActionName($id,$type,$name,$DATABASE['TABLE']);
        $skinName = strtoupper($id.'_'.$name."_".$DATABASE['TABLE']);
        $dest     = strtolower($action.'.html');

        if (!empty($src)) {

            { /* create GUI File */    
                $guiTemplate = new smartTemplate("$tplDir$src");
                $guiTemplate->insert('*',$INPUT);
                $guiTemplate->insert('*',$DATABASE);
                $guiTemplate->insert('ACTION',$action);
                $guiTemplate->insert('ON_NEW',$id."_write_new_".$DATABASE['TABLE']);
                $guiTemplate->insert('ON_EDIT',$id."_write_edit_".$DATABASE['TABLE']);
                $guiTemplate->insert('ON_DELETE',$id."_write_delete_".$DATABASE['TABLE']);
                $guiTemplate->insert('ON_SEARCH',$id."_read_search_".$DATABASE['TABLE']);
                $html =  $guiTemplate->get();
                $file = new secureFileStream($sknDir->getFilename().$dest);
                if (!$file->exists()) {
                    $file->create();
                    $file->insert($html);
                    $file->write() || die('<h1>Fehler</h1><p>Die Datei '.$file->getFilename().' konnte nicht gespeichert werden.</p>');
                }
            }

            { /* create Interface Entry */
                $INTERFACE[$action]['TYPE']       = $type;
                $INTERFACE[$action]['MODE']       = 0;
                $INTERFACE[$action]['INSERT']     = $skinName;
                $INTERFACE[$action]['PERMISSION'] = ($mode > 0) ? $mode : 0;
                $INTERFACE[$action]['ONSUCCESS'] = array();
                $INTERFACE[$action]['ONERROR'] = array();
                $INTERFACE[$action]['ONSUCCESS']['GOTO'] = $action;
                $INTERFACE[$action]['ONERROR']['GOTO'] = $action;
            }

            { /* create Skin Entry */
                $SKIN[$skinName] = array('FILE' => $id.'/'.$dest);
            }

            { /* create Root entry */
                if (($type == "default" || $type == "read") && $mode == 0) {
                    $INFO['START']  = $action;
                }
            }

        } else {
            
            { /* create Interface Entry */
                $INTERFACE[$action]['TYPE']       = $type;
                $INTERFACE[$action]['MODE']       = 0;
                $INTERFACE[$action]['TEMPLATE']   = 'MESSAGE';
                $INTERFACE[$action]['PERMISSION'] = ($mode > 0) ? $mode : 0;
                $INTERFACE[$action]['ONSUCCESS'] = array();
                $INTERFACE[$action]['ONERROR'] = array();
                $INTERFACE[$action]['ONSUCCESS']['GOTO'] = $action;
                $INTERFACE[$action]['ONERROR']['GOTO'] = $action;
            }

            switch ($name)
            {
                case 'edit':
                    $INTERFACE[$action]['ONSUCCESS']['GOTO'] = $this->_createActionName($id, 'read', $name, $DATABASE['TABLE']);
                    $INTERFACE[$action]['ONERROR']['GOTO'] = $this->_createActionName($id, 'read', $name, $DATABASE['TABLE']);
                break;
                case 'delete':
                    $INTERFACE[$action]['ONSUCCESS']['GOTO'] = $this->_createActionName($id, 'read', 'edit', $DATABASE['TABLE']);
                    $INTERFACE[$action]['ONERROR']['GOTO'] = $this->_createActionName($id, 'read', 'edit', $DATABASE['TABLE']);
                break;
                case 'new':
                    $INTERFACE[$action]['ONSUCCESS']['GOTO'] = $this->_createActionName($id, 'read', 'read', $DATABASE['TABLE']);
                    $INTERFACE[$action]['ONERROR']['GOTO'] = $this->_createActionName($id, 'default', $name, $DATABASE['TABLE']);
                break;
            }

        }

        $AUTO_INTERFACE[$action] = $INTERFACE[$action];
        $AUTO_INTERFACE[$action]['TABLE']  = $DATABASE['TABLE'];
        $AUTO_INTERFACE[$action]['PRESET'] = strtolower($name);
        return $action;
    }

    /**
     * @access private
     * @param scalar $id
     * @param scalar $type
     * @param scalar $name
     * @param scalar $table
     * @return string
     * @ignore
     */
    function _createActionName($id, $type, $name, $table)
    {
        return $id."_".$type."_".$name."_".$table;
    }
    
    /**
     * Backup current settings
     *
     * @access private
     * @param array $ARGS
     * @return bool
     * @ignore
     */
    function _createBackup($ARGS)
    {
        global $YANA;
        $config = $YANA->plugins->get($this->name, 'CONFIGFILE');
        $config->read();
        if (!$config->exists()) {
            $config->create();
        }
        return $config->insert($ARGS) && $config->write();
    }

}

//  2006, Thomas Meyer www.All-Community.de
// Copyright Hinweis nicht entfernen!
?>